/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.ide.ui.internal.actions;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPage;

import com.ibm.team.filesystem.client.FileSystemException;
import com.ibm.team.filesystem.client.IShareable;
import com.ibm.team.filesystem.client.ResourceType;
import com.ibm.team.filesystem.ui.changes.actions.OpenLocalFileAction;
import com.ibm.team.internal.filesystem.ui.Messages;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.rcp.ui.operations.IStatusCollector;
import com.ibm.team.repository.rcp.ui.operations.OperationFailedException;
import com.ibm.team.repository.rcp.ui.operations.RepositoryOperation;
import com.ibm.team.repository.rcp.ui.utils.AbstractActionDelegate;
import com.ibm.team.repository.rcp.ui.utils.UIContext;


public abstract class AbstractOpenLocalFileAction extends AbstractActionDelegate {

    @Override
    public void run(Shell shell, IWorkbenchPage page, final IStructuredSelection selection) {
        
        final UIContext context = getContext();
        getOperationRunner().enqueue(Messages.OpenLocalFileAction_0, new RepositoryOperation() {
            @Override
            public void repositoryRun(IProgressMonitor monitor, IStatusCollector problems)
                    throws TeamRepositoryException, FileSystemException, OperationFailedException {
                SubMonitor progress = SubMonitor.convert(monitor, 100);
                final Collection<IShareable> shareables = new ArrayList<IShareable>(getShareablesFor(selection, progress.newChild(50)));
                progress.setWorkRemaining(shareables.size());

                for (Iterator<IShareable> iShareables = shareables.iterator(); iShareables.hasNext(); ) {
                    IShareable shareable = iShareables.next();
                    if (shareable.getResourceType(progress.newChild(1)) != ResourceType.FILE) {
                        iShareables.remove();
                    }
                }
                context.asyncExec(new Runnable() {
                    public void run() {
                        if (!shareables.isEmpty()) {
                            for (IShareable shareable : shareables) {
                                OpenLocalFileAction.openLocalFile(context.getShell(), context.getPage(), shareable);
                            }
                        } else {
                            MessageDialog.openInformation(context.getShell(), Messages.OpenLocalFileAction_100, Messages.OpenLocalFileAction_200);
                        }
                    }
                });
             }
        });
    }
    
    /**
     * Convert the selection to a list of IShareables to open the local editor.
     */
    protected abstract Collection<IShareable> getShareablesFor(
            IStructuredSelection selection, IProgressMonitor monitor)
            throws FileSystemException, TeamRepositoryException;

}
