/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2012. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.internal.ide.java.ui;

import java.io.CharArrayWriter;
import java.io.IOException;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;
import org.w3c.dom.Document;

import com.ibm.team.filesystem.internal.ide.java.ui.deliver.TestRunManager;

/**
 * The activator class controls the plug-in life cycle
 */
public class DefinitionsIdeUIPlugin extends AbstractUIPlugin {

	// The plug-in ID
	public static final String PLUGIN_ID = "com.ibm.team.process.definitions.ide.ui"; //$NON-NLS-1$

	// The shared instance
	private static DefinitionsIdeUIPlugin plugin;
	
	private TestRunManager fTestRunManager;
	
	/**
	 * The constructor
	 */
	public DefinitionsIdeUIPlugin() {
		plugin = this;
	}

	/*
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#start(org.osgi.framework.BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
		TestRunManager manager = getTestRunManager();
		manager.start();
	}
	
	/*
	 * @see org.eclipse.ui.plugin.AbstractUIPlugin#stop(org.osgi.framework.BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
		if (fTestRunManager != null) {
			fTestRunManager.stop();
			fTestRunManager = null;
		}
		plugin = null;
		super.stop(context);
	}

	/**
	 * Returns the shared instance
	 *
	 * @return the shared instance
	 */
	public static DefinitionsIdeUIPlugin getDefault() {
		return plugin;
	}
	
	public void log(Throwable exception) {
		log(Messages.DefinitionsIdeUIPlugin_0, exception);
	}
	
	public void log(String message, Throwable exception) {
		log(new Status(IStatus.ERROR, PLUGIN_ID, 0, message, exception));
	}

	public void log(IStatus status) {
		getLog().log(status);
	}
	
	public TestRunManager getTestRunManager() {
		if (fTestRunManager == null) {
			fTestRunManager = new TestRunManager();
		}
		return fTestRunManager;
	}
	
	/**
	 * Returns an image descriptor for the image file at the given plug-in
	 * relative path.
	 * 
	 * @param path the path
	 * @return the image descriptor
	 */
	public static ImageDescriptor getImageDescriptor(String path) {
		return AbstractUIPlugin.imageDescriptorFromPlugin(PLUGIN_ID, path);
	}
	
	public static String serializeDocument(Document doc) throws TransformerException, IOException {

		// http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6296446
		// setting the indent amount property doesn't work as it should, the trick is to 
		// to set the property on the factory and to wrap the output stream with a writer
		
		TransformerFactory factory = TransformerFactory.newInstance();
		try {
			factory.setAttribute("indent-number", new Integer(4)); //$NON-NLS-1$
		} catch (IllegalArgumentException x) {
			// ignore
		}

		Transformer transformer = factory.newTransformer();
		transformer.setOutputProperty(OutputKeys.METHOD, "xml"); //$NON-NLS-1$
		transformer.setOutputProperty(OutputKeys.INDENT, "yes"); //$NON-NLS-1$
		try {
			transformer.setOutputProperty("{http://xml.apache.org/xalan}indent-amount", "4"); //$NON-NLS-1$ //$NON-NLS-2$
			transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "4"); //$NON-NLS-1$ //$NON-NLS-2$
		} catch (IllegalArgumentException x) {
			// ignore
		}

		CharArrayWriter s = new CharArrayWriter();
		StreamResult outputTarget = new StreamResult(s);
		DOMSource source = new DOMSource(doc);
		transformer.transform(source, outputTarget);

		return s.toString();	
	}
}
