/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.internal.ide.java.ui.deliver;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jdt.core.ICompilationUnit;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.ui.actions.ExternalizeStringsAction;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.IWorkbenchSite;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import com.ibm.team.process.client.IAdvisorProblemResolutionDelegate;
import com.ibm.team.process.common.advice.IAdvisorInfo;

/**
 * 
 */
public class ExternalizeStringsProblemResolutionDelegate implements IAdvisorProblemResolutionDelegate {

	/*
	 * @see com.ibm.team.process.client.IAdvisorProblemResolutionDelegate#run(com.ibm.team.process.common.advice.IAdvisorInfo, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public boolean run(final IAdvisorInfo problem, final IProgressMonitor monitor) {
		List filesWithNonexternalizedStrings = (List) problem.getProblemObject();
    	final List compilationUnits = new ArrayList();
    	for (Iterator iterator = filesWithNonexternalizedStrings.iterator(); iterator.hasNext();) {
			IFile file = (IFile) iterator.next();
            IJavaElement javaElement = JavaCore.create(file);
            if (javaElement instanceof ICompilationUnit) {
            	compilationUnits.add(javaElement);
            }
		}
        final boolean[] result = new boolean[] { false };
        if (compilationUnits.size() > 0) {
			Display.getDefault().syncExec(new Runnable() {
	            public void run() {
	            	IWorkbenchSite workbenchSite = getWorkbenchSite();
	            	if (workbenchSite != null) {
	            		ExternalizeStringsAction action = new ExternalizeStringsAction(workbenchSite);
	            		action.run(new StructuredSelection(compilationUnits));
		            	result[0] = true;
	            	}
	            }
	        });
        }
		return result[0];
	}
	
	/*
	 * @see com.ibm.team.process.client.IAdvisorProblemResolutionDelegate#mayBeApplicable(com.ibm.team.process.common.advice.IAdvisorInfo)
	 */
	public boolean mayBeApplicable(IAdvisorInfo info) {
		Object problemObject = info.getProblemObject();
		return (problemObject instanceof List && ((List) problemObject).size() > 0);
	}
	
	private IWorkbenchSite getWorkbenchSite() {
		IWorkbenchSite workbenchSite = null;
		IWorkbench workbench = PlatformUI.getWorkbench();
		IWorkbenchWindow window = workbench.getActiveWorkbenchWindow();
		if (window != null) {
			workbenchSite = getWorkbenchSite(window);
		}
		IWorkbenchWindow[] windows = workbench.getWorkbenchWindows();
		for (int i = 0; workbenchSite == null && i < windows.length; i++) {
			workbenchSite = getWorkbenchSite(windows[i]);
		}
    	return workbenchSite;
	}
	
	private IWorkbenchSite getWorkbenchSite(IWorkbenchWindow window) {
		IWorkbenchSite workbenchSite = null;
		IWorkbenchPage page = window.getActivePage();
		if (page != null) {
			workbenchSite = getWorkbenchSite(page);
		}
		IWorkbenchPage[] pages = window.getPages();
		for (int i = 0; workbenchSite == null && i < pages.length; i++) {
			workbenchSite = getWorkbenchSite(pages[i]);
		}
		return workbenchSite;
	}
	
	private IWorkbenchSite getWorkbenchSite(IWorkbenchPage page) {
		IWorkbenchPart part = page.getActivePart();
		if (part != null) {
			return part.getSite();
		}
		part = page.getActiveEditor();
		if (part != null) {
			return part.getSite();
		}
		IViewPart[] views = page.getViews();
		if (views.length > 0) {
			return views[0].getViewSite();
		}
		return null;
	}

}
