/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.internal.ide.java.ui.deliver;

import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.osgi.util.NLS;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.part.ISetSelectionTarget;

import com.ibm.team.filesystem.ide.ui.process.AbstractFileAdvisorDetailProvider;
import com.ibm.team.filesystem.internal.ide.java.ui.DefinitionsIdeUIPlugin;
import com.ibm.team.process.common.advice.IAdvisorInfo;

/**
 * Detail provider for problems created by the 
 * {@link ProhibitCompilationErrorsAdvisor}.
 */
public class ProhibitCompilationErrorsDetailProvider extends AbstractFileAdvisorDetailProvider {

	public ProhibitCompilationErrorsDetailProvider(IAdvisorInfo info) {
		super(info);
	}
	
	/**
	 * @param info
	 */
	protected void appendProblem(IAdvisorInfo info, StringBuffer buffer, IProgressMonitor monitor) {
		List projectsWithErrors = (List) info.getProblemObject();
		StringBuffer linkBuffer = new StringBuffer();
		appendHyperlink(Messages.ProhibitCompilationErrorsDetailProvider_0, Messages.ProhibitCompilationErrorsDetailProvider_1, linkBuffer, new Runnable() {
			public void run() {
				IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
				if (window != null) {
					IWorkbenchPage page = window.getActivePage();
					if (page != null) {
						try {
							page.showView("org.eclipse.ui.views.ProblemView"); //$NON-NLS-1$
						} catch (PartInitException e) {
							DefinitionsIdeUIPlugin.getDefault().log(e);
						}
					}
				}
			}
		});
		buffer.append(NLS.bind(Messages.ProhibitCompilationErrorsDetailProvider_2, linkBuffer));
		for (Iterator iterator = projectsWithErrors.iterator(); iterator.hasNext();) {
			final IProject project = (IProject) iterator.next();
			buffer.append("<br>&nbsp;&nbsp;"); //$NON-NLS-1$
			appendHyperlink(project.getName(), Messages.ProhibitCompilationErrorsDetailProvider_3, buffer, new Runnable() {
				public void run() {
					showProject(project);
				}
			});
		}
	}

	/**
	 * Attempts to show the project with the given name in the Resource Navigator.
	 */
	private void showProject(final IProject project) {
		if (project != null) {
			IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
			if (window == null) {
				IWorkbenchWindow[] workbenchWindows = PlatformUI.getWorkbench().getWorkbenchWindows();
				if (workbenchWindows.length > 0) {
					window = workbenchWindows[0];
				}
			}
			if (window != null) {
				final IWorkbenchPage page = window.getActivePage();
				if (page != null) {
					window.getShell().getDisplay().asyncExec(new Runnable() {
						public void run() {
							try {
								IViewPart view = page.showView("org.eclipse.ui.views.ResourceNavigator"); //$NON-NLS-1$
								if (view instanceof ISetSelectionTarget) {
									((ISetSelectionTarget) view).selectReveal(new StructuredSelection(project));
								}
							} catch (PartInitException e) {
								DefinitionsIdeUIPlugin.getDefault().log(e);
							}
						}
					});
				}
			}
		}
	}

}
