/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.internal.ide.java.ui.deliver;

import java.text.Collator;
import java.text.RuleBasedCollator;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.filesystem.ide.ui.process.AbstractFileAdvisorDetailProvider;
import com.ibm.team.process.common.advice.IAdvisorInfo;

/**
 * Detail provider for problems created by the 
 * {@link ProhibitNonexternalizedStringsAdvisor}.
 */
public class ProhibitNonexternalizedStringsDetailProvider extends AbstractFileAdvisorDetailProvider {

	public ProhibitNonexternalizedStringsDetailProvider(IAdvisorInfo info) {
		super(info);
	}
	
	protected void appendProblem(IAdvisorInfo info, StringBuffer buffer, IProgressMonitor monitor) {
		final List filesWithNonexternalizedStrings = (List) info.getProblemObject();
		Collections.sort(filesWithNonexternalizedStrings, new Comparator() {
			public int compare(Object arg0, Object arg1) {
				if (arg0 instanceof IFile && arg1 instanceof IFile) {
					RuleBasedCollator collator = (RuleBasedCollator)Collator.getInstance();
					return collator.compare(((IFile) arg0).getName(), ((IFile) arg1).getName());
				}
				return 0;
			}
		});
		int numFiles = filesWithNonexternalizedStrings.size();
		if (numFiles > 0) {
			int maxFilesToShow = 10;
			if (numFiles == 1) {
				buffer.append(Messages.ProhibitNonexternalizedStringsDetailProvider_0);
			} else {
				buffer.append(NLS.bind(Messages.ProhibitNonexternalizedStringsDetailProvider_1, Integer.valueOf(numFiles)));
			}
			for (int i = 0; i < filesWithNonexternalizedStrings.size() && i < maxFilesToShow; i++) {
				IFile file = (IFile) filesWithNonexternalizedStrings.get(i);
				buffer.append("<br>&nbsp;&nbsp;"); //$NON-NLS-1$
				appendFileLink(buffer, file);
			}
			if (numFiles > maxFilesToShow) {
				buffer.append("<br>&nbsp;&nbsp;"); //$NON-NLS-1$
				appendHyperlink(NLS.bind(Messages.ProhibitNonexternalizedStringsDetailProvider_2, Integer.valueOf(numFiles - maxFilesToShow)), Messages.ProhibitNonexternalizedStringsDetailProvider_3, buffer, new Runnable() {
					public void run() {
						showAllFiles(filesWithNonexternalizedStrings);
					}
				});
			}
		}
	}

}
