/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.internal.ide.java.ui.deliver;

import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.filesystem.internal.ide.java.ui.DefinitionsIdeUIPlugin;
import com.ibm.team.process.common.IProcessConfigurationElement;
import com.ibm.team.process.common.advice.AdvisableOperation;
import com.ibm.team.process.common.advice.IAdvisorInfo;
import com.ibm.team.process.common.advice.IAdvisorInfoCollector;
import com.ibm.team.process.common.advice.IProcessReport;
import com.ibm.team.process.common.advice.IReportInfo;
import com.ibm.team.process.common.advice.runtime.IOperationAdvisor;
import com.ibm.team.repository.common.TeamRepositoryException;


public class RequireTestRunAdvisor implements IOperationAdvisor {
	
	/**
	 * Process configuration element constants.
	 */
	private static final String TAG_SUITE = "suite"; //$NON-NLS-1$
	private static final String ATTR_CLASS = "class"; //$NON-NLS-1$
	
	public static final String ID_PROBLEM_TESTS_UNDEFINED = "com.ibm.team.process.definitions.testsUndefined"; //$NON-NLS-1$
    public static final String ID_PROBLEM_TESTS_NOT_RUN = "com.ibm.team.process.definitions.testsNotRunProblem"; //$NON-NLS-1$
    public static final String ID_PROBLEM_FAILED_TESTS = "com.ibm.team.process.definitions.failedTestsProblem"; //$NON-NLS-1$
    
    public static final String ID_REQUIRE_TEST_RUN = "com.ibm.team.process.deliver.requireTestRun"; //$NON-NLS-1$
    
    private String fTestSuite;

	public RequireTestRunAdvisor() {
		super();
	}
	
	/**
	 * Initializes this advisor from the given configuration.
	 */
	private void initialize(IProcessConfigurationElement element) {
		IProcessConfigurationElement[] children = element.getChildren();
		for (int i = 0; i < children.length; i++) {
			IProcessConfigurationElement child = children[i];
			String elementName = child.getName();
			if (elementName.equals(TAG_SUITE)) {
				fTestSuite = child.getAttribute(ATTR_CLASS);
			}
		}
	}
	
	/*
	 * @see com.ibm.team.process.common.advice.runtime.IOperationAdvisor#run(com.ibm.team.process.common.advice.AdvisableOperation, com.ibm.team.process.common.IProcessConfigurationElement, com.ibm.team.process.common.advice.IAdvisorInfoCollector, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void run(AdvisableOperation operation, IProcessConfigurationElement advisorConfiguration, IAdvisorInfoCollector collector, IProgressMonitor monitor) throws TeamRepositoryException {
		initialize(advisorConfiguration);
		if (fTestSuite == null || fTestSuite.trim().length() == 0) {
			String description = Messages.RequireTestRunAdvisor_TESTS_SHOULD_BE_RUN_LOCALLY; 
			IReportInfo problem = collector.createProblemInfo(Messages.RequireTestRunAdvisor_0, description, ID_PROBLEM_TESTS_UNDEFINED);
			problem.setSeverity(IProcessReport.ERROR);
			collector.addInfo(problem);
			return;
		}
		
		TestRunManager testRunManager = DefinitionsIdeUIPlugin.getDefault().getTestRunManager();
		
		if (!testRunManager.wereTestsRun(fTestSuite)) {
			String description = Messages.RequireTestRunAdvisor_1;
			IAdvisorInfo problem = collector.createProblemInfo(Messages.RequireTestRunAdvisor_2, description, ID_PROBLEM_TESTS_NOT_RUN);
			problem.setData(fTestSuite);
			collector.addInfo(problem);
		} else {
			Map failedTests = testRunManager.getFailedTests(fTestSuite);
			if (failedTests != null && !failedTests.isEmpty()) {
				StringBuffer buffer = new StringBuffer(Messages.RequireTestRunAdvisor_3);
				buffer.append(Messages.RequireTestRunAdvisor_4);
				for (Iterator e = failedTests.values().iterator(); e.hasNext();) {
					buffer.append(e.next());
					buffer.append("\n"); //$NON-NLS-1$
				}
				IAdvisorInfo problem = collector.createProblemInfo(Messages.RequireTestRunAdvisor_5, buffer.toString(), ID_PROBLEM_FAILED_TESTS);
				problem.setData(fTestSuite);
				collector.addInfo(problem);
			}
		}
	}
}
