/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.internal.ide.java.ui.deliver.editors;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;

import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IType;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jdt.core.JavaModelException;
import org.eclipse.jdt.ui.JavaElementLabelProvider;
import org.eclipse.jface.fieldassist.ControlDecoration;
import org.eclipse.jface.fieldassist.FieldDecorationRegistry;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.operation.IRunnableContext;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.ElementListSelectionDialog;
import org.eclipse.ui.dialogs.SelectionDialog;
import org.eclipse.ui.forms.widgets.FormToolkit;

import com.ibm.team.filesystem.ide.ui.internal.util.DisplayHelper;
import com.ibm.team.foundation.common.util.IMemento;
import com.ibm.team.process.ide.ui.IProcessAspectEditorSite;
import com.ibm.team.process.ide.ui.OperationDetailsAspectEditor;
import com.ibm.team.process.ide.ui.ProcessAspect;
import com.ibm.team.process.internal.ide.ui.editors.form.util.MessageBox;

/**
 * Aspect editor for the details of the require test run precondition. 
 */
public class RequireTestRunAspectEditor extends OperationDetailsAspectEditor {
	
//    <suite class="classname"/>

	private static final String TAG_SUITE = "suite"; //$NON-NLS-1$
	private static final String ATTR_CLASS = "class"; //$NON-NLS-1$

	private static class Model {
		
		String className = ""; //$NON-NLS-1$
		
		public void readFrom(IMemento memento) {
			IMemento suiteMemento = memento.getChild(TAG_SUITE);
			if (suiteMemento != null) {
				className = normalize(suiteMemento.getString(ATTR_CLASS));
			}
		}
		
		public boolean saveTo(IMemento memento) {
			if (isNotNull(className)) {
				IMemento suiteMemento = memento.createChild(TAG_SUITE);
				suiteMemento.putString(ATTR_CLASS, className);
				return true;
			}
			return false;
		}
	}
	
	private static boolean isNotNull(String string) {
		return string != null && string.trim().length() > 0;
	}
		
	private static String normalize(String string) {
		return string == null ? "" : string.trim(); //$NON-NLS-1$
	}

	private Model fModel;
	private ArrayList fTestSuiteKinds;
	
	// SUPPRESS 63763
	@SuppressWarnings("restriction")
	private org.eclipse.jdt.internal.junit.launcher.ITestKind fTestKind;
	
	private IJavaProject fJavaProject;
	
	private Text fJavaProjectField;
	private Text fTestSuiteField;
	private ControlDecoration fTestSuiteDecoration;	
	private Button fSearchTestSuiteButton;
	private MessageBox fMessageBox;
	private boolean fSettingInput;

	
	public void createControl(Composite parent, FormToolkit toolkit) {
		GridLayoutFactory.fillDefaults().spacing(10, 8).numColumns(1).applyTo(parent);

		Label heading = toolkit.createLabel(parent, Messages.RequireTestRunAspectEditor_0);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(heading);
		heading.setFont(getBoldFont());
		
		Composite body = toolkit.createComposite(parent);
		GridDataFactory.fillDefaults().grab(true, true).applyTo(body);
		GridLayoutFactory.fillDefaults().spacing(10, 8).numColumns(3).applyTo(body);
		
		Label label = toolkit.createLabel(body, Messages.RequireTestRunAspectEditor_1);
		GridDataFactory.fillDefaults().applyTo(label);
		
		final Combo combo = new Combo(body, SWT.DROP_DOWN);
		GridDataFactory.fillDefaults().grab(true, false).span(2, 1).applyTo(combo);
		combo.setItems(getTestSuiteKindLabels());
		combo.select(0);
		fTestKind = getTestKindByIndex(0);
		combo.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				fTestKind = getTestKindByIndex(combo.getSelectionIndex());
			}
		});
		
		label = toolkit.createLabel(body, Messages.RequireTestRunAspectEditor_2);
		GridDataFactory.fillDefaults().applyTo(label);
		fJavaProjectField = toolkit.createText(body, "", SWT.SINGLE | SWT.BORDER); //$NON-NLS-1$
		GridDataFactory.fillDefaults().grab(true, false).applyTo(fJavaProjectField);
		fJavaProjectField.setEditable(false);
		fJavaProjectField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				fSearchTestSuiteButton.setEnabled(isNotNull(fJavaProjectField.getText()));
			}
		});
		
		Button selectProject = toolkit.createButton(body, Messages.RequireTestRunAspectEditor_3, SWT.FLAT);
		GridDataFactory.fillDefaults().grab(false, false).applyTo(fJavaProjectField);
		selectProject.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleSelectProject();
			}
		});
		
		label = toolkit.createLabel(body, Messages.RequireTestRunAspectEditor_4);
		GridDataFactory.fillDefaults().applyTo(label);
		fTestSuiteField = toolkit.createText(body, "", SWT.SINGLE | SWT.BORDER); //$NON-NLS-1$
		GridDataFactory.fillDefaults().grab(true, false).applyTo(fTestSuiteField);
		fTestSuiteField.setEditable(false);
		fTestSuiteDecoration = new ControlDecoration(fTestSuiteField, SWT.TOP | SWT.LEFT);
		fTestSuiteField.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				if (!fSettingInput) {
					setDirty();
					validateData();
				}
			}
		});
		
		fSearchTestSuiteButton = toolkit.createButton(body, Messages.RequireTestRunAspectEditor_5, SWT.FLAT);
		GridDataFactory.fillDefaults().grab(false, false).applyTo(fJavaProjectField);
		fSearchTestSuiteButton.setEnabled(false);
		fSearchTestSuiteButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				handleSearchTestSuite();
			}
		});
				
		fMessageBox = new MessageBox();
		Composite messageBoxControl = fMessageBox.createControl(parent, toolkit, this);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(messageBoxControl);
		
		DisplayHelper.asyncExec(messageBoxControl, new Runnable(){
			public void run() {
				updateUI();
			}
		});
	}
	
	private Font getBoldFont() {
		return JFaceResources.getFontRegistry().getBold(JFaceResources.DEFAULT_FONT);
	}
	
	// SUPPRESS 63763
	@SuppressWarnings("restriction")
	private String[] getTestSuiteKindLabels() {
		ArrayList labels = new ArrayList();
		for (Iterator e = fTestSuiteKinds.iterator(); e.hasNext();) {
			org.eclipse.jdt.internal.junit.launcher.ITestKind kind = (org.eclipse.jdt.internal.junit.launcher.ITestKind) e.next();
			labels.add(kind.getDisplayName());
		}
		return (String[]) labels.toArray(new String[labels.size()]);
	}

	/**
	 * Update the UI from the configuration.
	 */
	public void updateUI() {
		fSettingInput = true;
		try {
			fTestSuiteField.setText(fModel.className);
			validateData();
		} finally {
			fSettingInput = false;
		}
	}
	
	
	private void handleSelectProject() {
		fJavaProject = selectJavaProject();
		fJavaProjectField.setText(fJavaProject == null ? "" : fJavaProject.getElementName()); //$NON-NLS-1$
	}
	
	private IJavaProject selectJavaProject() {
		IJavaProject[] projects;
		try {
			IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
			projects= JavaCore.create(root).getJavaProjects();
		} catch (JavaModelException x) {
			projects= new IJavaProject[0];
		}
		
		ElementListSelectionDialog dialog= new ElementListSelectionDialog(fJavaProjectField.getShell(), new JavaElementLabelProvider(JavaElementLabelProvider.SHOW_DEFAULT));
		dialog.setTitle(Messages.RequireTestRunAspectEditor_6); 
		dialog.setMessage(Messages.RequireTestRunAspectEditor_7); 
		dialog.setElements(projects);
		
		if (dialog.open() == Window.OK) {			
			return (IJavaProject) dialog.getFirstResult();
		}			
		return null;		
	}
	
	private void handleSearchTestSuite() {
		IType testSuite = selectTestSuite();
		if (testSuite != null) {
			fJavaProjectField.setText(testSuite.getJavaProject().getElementName());
			fModel.className = testSuite.getFullyQualifiedName();
			fTestSuiteField.setText(fModel.className);
		} else {
			fTestSuiteField.setText(""); //$NON-NLS-1$
		}
	}
	
	private IType selectTestSuite() {
		IType[] testSuites = getAllTestSuites();
		SelectionDialog dialog = new TestSelectionDialog(fTestSuiteField.getShell(), testSuites);
		dialog.setTitle(Messages.RequireTestRunAspectEditor_9); 
		dialog.setMessage(Messages.RequireTestRunAspectEditor_10); 
		if (dialog.open() == Window.OK) {
			Object[] results = dialog.getResult();
			if (results != null && results.length >= 1) {
				return (IType) results[0];
			}		
		}
		return null;
	}
	
	private IType[] getAllTestSuites() {
		ArrayList testSuites = new ArrayList();
		
		IWorkbenchWindow workbenchWindow = findWorkbenchWindow();		
		try {
			IType[] types= findTests(workbenchWindow, fJavaProject, fTestKind);
			if (types != null && types.length > 0) {
				testSuites.addAll(Arrays.asList(types));
			}
		} catch (InterruptedException x) {
		} catch (InvocationTargetException x) {
		}
		return (IType[]) testSuites.toArray(new IType[testSuites.size()]);
	}
	
	// SUPPRESS 63763
	@SuppressWarnings("restriction")
	private IType[] findTests(IRunnableContext context, IJavaElement element, org.eclipse.jdt.internal.junit.launcher.ITestKind testKind) throws InvocationTargetException, InterruptedException {
		return org.eclipse.jdt.internal.junit.util.TestSearchEngine.findTests(context, element, testKind);
	}
	
	// SUPPRESS 63763
	@SuppressWarnings("restriction")
	private org.eclipse.jdt.internal.junit.launcher.ITestKind getTestKindByIndex(int index) {
		return (org.eclipse.jdt.internal.junit.launcher.ITestKind) fTestSuiteKinds.get(index);
	}
	
	private IWorkbenchWindow findWorkbenchWindow() {		
		IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
		if (window == null) {
			IWorkbenchWindow[] workbenchWindows = PlatformUI.getWorkbench().getWorkbenchWindows();
			if (workbenchWindows.length > 0) {
				window = workbenchWindows[0];
			}
		}
		return window;
	}


	public void dispose() {
	}

	public void init(IProcessAspectEditorSite site, ProcessAspect inputAspect) {
		super.init(site, inputAspect);
		fModel = new Model();
		fTestSuiteKinds = getAllKinds();
	}

	// SUPPRESS 63763
	@SuppressWarnings("restriction")
	private ArrayList getAllKinds() {
		return org.eclipse.jdt.internal.junit.launcher.TestKindRegistry.getDefault().getAllKinds();
	}
	
	public void restoreState(IMemento memento) {
		fModel.readFrom(memento);
	}

	public boolean saveState(IMemento memento) {
		return fModel.saveTo(memento);
	}
	
	private void validateData() {
		if (validateTestSuite()) {
			fMessageBox.clearMessage();
		}
	}
	
	private boolean validateTestSuite() {
		String testSuiteName = fTestSuiteField.getText();
		if (isNull(testSuiteName)) {
			setValidationMessage(true, fTestSuiteDecoration, true, Messages.RequireTestRunAspectEditor_11);
			return false;
		}
		setValidationMessage(true, fTestSuiteDecoration, true, null);
		return true;
	}
	
	private static boolean isNull(String string) {
		return string == null || string.trim().length() == 0;
	}
	
	private void setValidationMessage(boolean required, ControlDecoration decoration, boolean setErrorMessage, String msg) {
		if (msg != null) {
			if (setErrorMessage) {
				fMessageBox.setErrorMessage(msg);
			}
			decoration.setImage(FieldDecorationRegistry.getDefault().getFieldDecoration(FieldDecorationRegistry.DEC_ERROR).getImage());
		} else {
			clearDecoration(decoration, required);
		}
	}
	
	private void clearDecoration(ControlDecoration decoration, boolean required) {
		if (required) {
			decoration.setImage(FieldDecorationRegistry.getDefault().getFieldDecoration(FieldDecorationRegistry.DEC_REQUIRED).getImage());
		} else {
			decoration.hide();
			decoration.setImage(null);
			decoration.show();
		}
	}	
}
