/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2005, 2014. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.internal.rcp.ui.workitems.actions;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPage;

import com.ibm.team.filesystem.client.FileSystemException;
import com.ibm.team.filesystem.client.workitems.IFileSystemWorkItemManager;
import com.ibm.team.filesystem.internal.rcp.ui.workitems.Messages;
import com.ibm.team.filesystem.rcp.core.internal.changes.model.ComponentSyncUtil;
import com.ibm.team.filesystem.rcp.core.internal.changes.model.IFileSystemWorkItem;
import com.ibm.team.links.common.ILink;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.rcp.ui.operations.IOperationRunner;
import com.ibm.team.repository.rcp.ui.operations.IStatusCollector;
import com.ibm.team.repository.rcp.ui.operations.OperationFailedException;
import com.ibm.team.repository.rcp.ui.operations.RepositoryOperation;
import com.ibm.team.repository.rcp.ui.utils.AbstractActionDelegate;

/**
 * Remove the selected work item(s) in Pending Changes view
 */
public class RemoveWorkItemAction extends AbstractActionDelegate {

	protected Map<ITeamRepository,List<ILink>> result;

	@Override
	public void run(Shell shell, IWorkbenchPage page,
			IStructuredSelection selection) {
		result = new HashMap<ITeamRepository, List<ILink>>();
		extractSelection(selection);

		if (result.size() != 0) {
			deleteWorkItem(shell, getOperationRunner(), result);
		}
	}

	public void selectionChanged(IAction action, ISelection selection) {
		super.selectionChanged(action, selection);
		action.setEnabled(true);
	}

	protected boolean extractSelection(ISelection selection) {
		boolean enable = false;
		List list = ((IStructuredSelection) selection).toList();
		Set contexts = ComponentSyncUtil.getContexts(list);
		Set[] filtered = ComponentSyncUtil.filter(list, new Class[] {
				IFileSystemWorkItem.class, Object.class });
		if (contexts.size() != 0 && ComponentSyncUtil.allLoggedIn(contexts)) {
			/* Connected */
			if (filtered[1].size() == 0) {
				/*
				 * The entire selection applies to this operation and is not
				 * empty
				 */
				if (filtered[0].size() != 0) {
					enable = true;
					for (Iterator<IFileSystemWorkItem> i = filtered[0].iterator(); i.hasNext();) {
						IFileSystemWorkItem item = i.next();
						Object repo = item.getWorkItemHandle().getOrigin();
						ILink link = item.getLink();
						
						if (repo instanceof ITeamRepository && link != null) {
							addLink(link, (ITeamRepository) repo);
						}
					}
				}
			}
		}
		return enable;
	}

	private void deleteWorkItem(Shell shell, IOperationRunner runner, final Map<ITeamRepository, List<ILink>> result2) {
		String dialogTitle = Messages.RemoveChangeSetReasonsActions_2;
		String dialogMessage = Messages.RemoveChangeSetReasonsActions_3; // plural message
		if (result2 != null && result2.size() == 1) {
			List<ILink> links = result2.entrySet().iterator().next().getValue();
			if (links != null && links.size() == 1) {
				dialogMessage = Messages.RemoveChangeSetReasonsActions_10; // singular message.
			}
		}

		if (!MessageDialog.openQuestion(shell, dialogTitle, dialogMessage)) {
			return;
		}

		runner.enqueue(Messages.RemoveChangeSetReasonsActions_4,
				new RepositoryOperation() {
					public void repositoryRun(IProgressMonitor monitor,
							IStatusCollector problems)
							throws TeamRepositoryException,
							FileSystemException, OperationFailedException {
						try {
							monitor.beginTask(
									Messages.RemoveChangeSetReasonsActions_4,
									result2.size());
							Set<Entry<ITeamRepository,List<ILink>>> entries = result2
									.entrySet();
							for (Entry<ITeamRepository, List<ILink>> entry : entries) {
								ITeamRepository repo = entry.getKey();
								if (entry.getValue() != null && !entry.getValue().isEmpty()) {
									IFileSystemWorkItemManager linkService = (IFileSystemWorkItemManager) repo
											.getClientLibrary(IFileSystemWorkItemManager.class);
									List<ILink> links = entry.getValue();
									linkService.deleteLinks(links, new SubProgressMonitor(monitor, 1));
								}
							}
						} finally {
							monitor.done();
						}
					}
				});
	}
	
	protected void addLink(ILink link, ITeamRepository repo) {
		List<ILink> links = result.get(repo);
		if (links == null) {
			links = new ArrayList<ILink>();
		}
		links.add(link);
		result.put((ITeamRepository) repo, links);
	}
}
