/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.internal.rcp.ui.workitems.advisor.editors;

import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.forms.widgets.FormToolkit;

import com.ibm.team.filesystem.common.workitems.internal.process.AddCommentChangeSetsParticipantConstants;
import com.ibm.team.filesystem.internal.rcp.ui.workitems.Messages;
import com.ibm.team.foundation.common.util.IMemento;
import com.ibm.team.process.ide.ui.OperationDetailsAspectEditor;

/**
 * Aspect editor for the details of the add comment to work item when a change
 * set is associated follow-up action.
 */
public class AddCommentChangeSetsAspectEditor extends OperationDetailsAspectEditor {
	private Button includeCommentButton;
	private Button includeDateButton;
	private Model fModel;
	
	private static class Model {
		boolean comment;
		boolean date;
		
		public void readFrom(IMemento memento) {
			comment = true;
			date = true;
			IMemento properties = memento.getChild(AddCommentChangeSetsParticipantConstants.ATTR_PROPERTIES);
			if (properties != null) {
				comment = Boolean.parseBoolean(properties.getString(AddCommentChangeSetsParticipantConstants.ATTR_COMMENT));
				date = Boolean.parseBoolean(properties.getString(AddCommentChangeSetsParticipantConstants.ATTR_DATE));
			}
		}
	}
	
	/**
	 * Creates an instance of the add comment on change set/work item association
	 * aspect editor.
	 */
	public AddCommentChangeSetsAspectEditor() {
		fModel = new Model();
	}

	private void updateUI() {
		includeCommentButton.setSelection(fModel.comment);
		includeDateButton.setSelection(fModel.date);
	}

	@Override
	public boolean saveState(IMemento memento) {
		IMemento properties = memento.createChild(AddCommentChangeSetsParticipantConstants.ATTR_PROPERTIES);
		properties.putString(AddCommentChangeSetsParticipantConstants.ATTR_COMMENT, Boolean.toString(includeCommentButton.getSelection()));
		properties.putString(AddCommentChangeSetsParticipantConstants.ATTR_DATE, Boolean.toString(includeDateButton.getSelection()));
		return true;
	}

	@Override
	public void restoreState(IMemento memento) {
		fModel.readFrom(memento);
	}

	@Override
	public void createControl(Composite parent, FormToolkit toolkit) {
		GridLayoutFactory.fillDefaults().spacing(5, 8).applyTo(parent);
		
		includeCommentButton = new Button(parent, SWT.CHECK);
		includeCommentButton.setText(Messages.AddCommentChangeSetsAspectEditor_editor_includeComment);
		includeCommentButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				fModel.comment = includeCommentButton.getSelection();
				setDirty();
			}
		});
		
		includeDateButton = new Button(parent, SWT.CHECK);
		includeDateButton.setText(Messages.AddCommentChangeSetsAspectEditor_editor_includeDate);
		includeDateButton.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				fModel.date = includeDateButton.getSelection();
				setDirty();
			}
		});

		updateUI();
	}

	@Override
	public void dispose() {
	}
}