/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.internal.rcp.ui.workitems.deliveradvisor;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.filesystem.common.workitems.internal.process.RequireWorkItemProblemObject;
import com.ibm.team.process.client.IProcessItemService;
import com.ibm.team.process.common.IDevelopmentLine;
import com.ibm.team.process.common.IIteration;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.workitem.common.model.IWorkItem;


public class AssignWorkItemTargetResolutionDelegate extends AbstractWorkItemResolutionDelegate {

	/*
	 * @see com.ibm.team.process.internal.definitions.ide.ui.deliver.AbstractWorkItemResolutionDelegate#mayBeApplicable(com.ibm.team.process.internal.definitions.ide.ui.deliver.RequireWorkItemProblemObject)
	 */
	public boolean mayBeApplicable(RequireWorkItemProblemObject problemObject) {
		return problemObject.isNeedTarget();
	}
	
	/*
	 * @see com.ibm.team.process.internal.definitions.ide.ui.deliver.AbstractWorkItemResolutionDelegate#modifyWorkItem(com.ibm.team.workitem.common.model.IWorkItem, com.ibm.team.process.common.IDevelopmentLine, com.ibm.team.repository.client.ITeamRepository, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void modifyWorkItem(IWorkItem item, IDevelopmentLine devLine, ITeamRepository teamRepository, IProgressMonitor monitor) throws TeamRepositoryException {
		if (devLine != null) {
			// Walk the chain of iterations starting from the current and proceeding to the parent.
			// Assign the work item to the first iteration which has a deliverable.
			IProcessItemService processService = (IProcessItemService) teamRepository.getClientLibrary(IProcessItemService.class);
			IIteration[] currentIterations = processService.getCurrentIterations(devLine, monitor);
			for (int i = currentIterations.length - 1; i >= 0; i--) {
				IIteration iteration = currentIterations[i];
				if (iteration.hasDeliverable()) {
					item.setTarget(iteration);
					return;
				}
			}
			// If no iteration was found with a deliverable, then simply assign to the current iteration.
			item.setTarget(devLine.getCurrentIteration());
		}
	}
}
