/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2006, 2013. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.internal.rcp.ui.workitems.deliveradvisor;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

import com.ibm.team.filesystem.client.internal.utils.RepositoryUtils;
import com.ibm.team.filesystem.client.workitems.IFileSystemWorkItemManager;
import com.ibm.team.filesystem.common.workitems.internal.process.RequireWorkItemProblemObject;
import com.ibm.team.filesystem.ide.ui.process.providers.ParsedUnifiedDescriptionRegistry;
import com.ibm.team.process.client.IAdvisorProblemResolutionDelegate;
import com.ibm.team.process.common.IProjectAreaHandle;
import com.ibm.team.process.common.advice.IAdvisorInfo;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.rcp.core.utils.StatusUtil;
import com.ibm.team.scm.common.internal.process.AdvisorDataMarshaller.AdvisorData;
import com.ibm.team.scm.common.internal.process.AdvisorDataMarshaller.IScmAdvisorMarshallable;
import com.ibm.team.scm.common.internal.process.AdvisorDataMarshaller.MarshallerException;
import com.ibm.team.workitem.common.model.IWorkItemHandle;
import com.ibm.team.workitem.rcp.ui.workitempicker.WorkItemSelectionDialog;

/**
 * Problem resolution delegate which prompts the user to associate
 * comments/workitems with a change set.
 */
public class AssociateWorkItemResolutionDelegate implements IAdvisorProblemResolutionDelegate {

	/*
	 * @see com.ibm.team.process.client.IAdvisorProblemResolutionDelegate#run(com.ibm.team.process.common.advice.IAdvisorInfo, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public boolean run(IAdvisorInfo problem, IProgressMonitor monitor) {
		Object object;
		try {
			AdvisorData<IScmAdvisorMarshallable> data = ParsedUnifiedDescriptionRegistry.getInstance().parse(problem);
			if (data == null) {
				return false;
			}
			
			object = data.data;
		} catch (MarshallerException e1) {
			return false;
		}
		
		final boolean[] result = new boolean[] { false };
		if (object instanceof RequireWorkItemProblemObject) {
			final RequireWorkItemProblemObject problemObject = (RequireWorkItemProblemObject) object;
	        final Display display = Display.getDefault();
			display.syncExec(new Runnable() {
				public void run() {
			        final IWorkItemHandle[] workItems = openWorkItemPicker(display.getActiveShell(), problemObject.getProjectArea());
			        if (workItems.length > 0) {
                        result[0] = true;
			            Job job = new Job(Messages.AssociateWorkItemResolutionDelegate_0) {
			                protected IStatus run(IProgressMonitor monitor) {
			                    monitor.beginTask(Messages.AssociateWorkItemResolutionDelegate_1, 1000);
			                    try {
			                    	ITeamRepository repo = RepositoryUtils.getTeamRepositoryById(problemObject.getRepoUuid());
			                    	IFileSystemWorkItemManager workItemManager = (IFileSystemWorkItemManager) repo.getClientLibrary(IFileSystemWorkItemManager.class);
			                    	workItemManager.createLink(
                                            problemObject.getSourceWorkspace(), 
                                            problemObject.getChangeSet(), 
                                            workItems, 
                                            monitor);
			                        return Status.OK_STATUS;
			                    } catch (TeamRepositoryException e) {
			                        return StatusUtil.newStatus(this, Messages.AssociateWorkItemResolutionDelegate_2, e);
			                    } finally {
			                    	monitor.done();
			                    }
			                }
			            };
			            job.setUser(true);
			            job.schedule();
			        }
				}
			});
		}
		return result[0];
	}
	
	/*
	 * @see com.ibm.team.process.client.IAdvisorProblemResolutionDelegate#mayBeApplicable(com.ibm.team.process.common.advice.IAdvisorInfo)
	 */
	public boolean mayBeApplicable(IAdvisorInfo info) {
		if (RequireWorkItemProblemObject.ID_REQUIRE_WORK_ITEM_PROBLEM.equals(info.getIdentifier())) {
			Object object;
			try {
				AdvisorData<IScmAdvisorMarshallable> parsed = ParsedUnifiedDescriptionRegistry.getInstance().parse(info);
				if (parsed == null) {
					return false;
				}

				object = parsed.data;
			} catch (MarshallerException e1) {
				return false;
			}
			
			if (! (object instanceof RequireWorkItemProblemObject)) {
				return false;
			}
			
			return object != null && ((RequireWorkItemProblemObject)object).wantsWorkItem();
		}
		return false;
	}

    /**
     * Prompts the user to select a workitem.
     */
    public static IWorkItemHandle[] openWorkItemPicker(Shell parent, IProjectAreaHandle projectArea) {
        WorkItemSelectionDialog dialog = new WorkItemSelectionDialog(parent, projectArea, WorkItemSelectionDialog.MULTIPLE_SELECTION);
        dialog.setTitle(Messages.AssociateWorkItemResolutionDelegate_3);
        if (Window.OK == dialog.open()) {
            return (IWorkItemHandle[]) dialog.getResult();
        }
        return new IWorkItemHandle[0];
    }
}