/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.filesystem.internal.rcp.ui.workitems.deliveradvisor;

import java.util.Iterator;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import com.ibm.team.filesystem.common.workitems.internal.process.RequireWorkItemProblemObject;
import com.ibm.team.process.client.IAdvisorProblemResolutionDelegate;
import com.ibm.team.process.common.advice.IAdvisorInfo;
import com.ibm.team.workitem.common.model.IWorkItemHandle;
import com.ibm.team.workitem.rcp.ui.WorkItemUI;

/**
 * Problem resolution which opens all workitems associated with a change set so
 * the user can manually edit them to meet requirements.
 */
public class OpenAssociatedWorkItemsResolutionDelegate implements IAdvisorProblemResolutionDelegate {

	/*
	 * @see com.ibm.team.process.client.IAdvisorProblemResolutionDelegate#run(com.ibm.team.process.common.advice.IAdvisorInfo, org.eclipse.core.runtime.IProgressMonitor)
	 */
	public boolean run(IAdvisorInfo info, IProgressMonitor monitor) {
		Object problemObject = info.getProblemObject();
		final boolean[] result = new boolean[] { false };
		if (problemObject instanceof RequireWorkItemProblemObject) {
	        final Display display = Display.getDefault();
	        final List workItems = ((RequireWorkItemProblemObject) problemObject).getWorkItems();
			display.syncExec(new Runnable() {
				public void run() {
					IWorkbenchWindow window = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
					if (window != null) {
						IWorkbenchPage page = window.getActivePage();
						if (page != null) {
							for (Iterator iter = workItems.iterator(); iter.hasNext();) {
								IWorkItemHandle workitem = (IWorkItemHandle) iter.next();
								if (WorkItemUI.openEditor(page, workitem) != null) {
									result[0] = true;
								}
							}
						}
					}
				}
			});
		}
		return result[0];
	}
	
	/*
	 * @see com.ibm.team.process.client.IAdvisorProblemResolutionDelegate#mayBeApplicable(com.ibm.team.process.common.advice.IAdvisorInfo)
	 */
	public boolean mayBeApplicable(IAdvisorInfo info) {
		Object object = info.getProblemObject();
		if (object instanceof RequireWorkItemProblemObject) {
			// This resolution is not applicable if the problem is a missing comment
			RequireWorkItemProblemObject problemObject = (RequireWorkItemProblemObject) object;
			return problemObject.isNeedOwner() || problemObject.isNeedTarget();
		}
		return false;
	}

}
