/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.metronome.internal;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.ui.plugin.AbstractUIPlugin;

import com.ibm.team.filesystem.client.internal.MetronomeModel;
import com.ibm.team.metronome.MetronomePlugin;

public class MetronomeControl extends Composite {

	private MetronomeModel metronomeModel;

	private Image musicalNoteImage;

	private MetronomeRepositorySelectionToolItem repositorySelectionItem;

	public MetronomeControl(final Composite parent) {
		super(parent, SWT.NONE);

		metronomeModel = MetronomeModel.getInstance();
		addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				musicalNoteImage = null;
			}
		});

		ImageDescriptor descr = AbstractUIPlugin.imageDescriptorFromPlugin(
				MetronomePlugin.PLUGIN_ID, "icons/obj16/metronome.gif"); //$NON-NLS-1$
		musicalNoteImage = descr.createImage();
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		layout.marginWidth = 0;
		layout.marginHeight = 0;
		layout.horizontalSpacing = 5;
		layout.verticalSpacing = 0;
		setLayout(layout);

		ToolBar toolBar = new ToolBar(this, SWT.RIGHT | SWT.FLAT);
		toolBar.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, true));

		repositorySelectionItem = new MetronomeRepositorySelectionToolItem(
				toolBar, SWT.NONE, metronomeModel);
		final ToolItem openMetronomeToolItem = repositorySelectionItem
				.getControl();
		openMetronomeToolItem.setImage(musicalNoteImage);
		openMetronomeToolItem.setToolTipText(Messages.openButtonTooltip);
		openMetronomeToolItem.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				openMetronomeToolItem.getImage().dispose();
			}
		});

		toolBar.pack();
	}
}
