/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.metronome.internal;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.preferences.InstanceScope;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

import com.ibm.team.filesystem.client.internal.MetronomeModel;
import com.ibm.team.filesystem.ui.UiPlugin;
import com.ibm.team.jface.labelProviders.ElementRemovedNotifierImpl;
import com.ibm.team.jface.labelProviders.StandardLabelProvider;
import com.ibm.team.repository.client.IStatistics;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.client.TeamPlatform;

/**
 * Dropdown in the Metronome trim item.
 */
public class MetronomeRepositorySelectionToolItem {

	private Map<IStatistics, MetronomeWindow> mapMetronomeWindows;

	MetronomeModel metronomeModel;

	private ToolItem fItem;

	private Menu fMenu;
	
	private ITeamRepository defaultRepo;

	private Listener fPopupListener = new Listener() {
		public void handleEvent(Event event) {
			if (event.detail != 4 && defaultRepo != null) {
				openMetronomeWindow(defaultRepo
						.statistics(), defaultRepo.getRepositoryURI());
			} else {
				showPopupMenu(event);
			}
		}
	};

	private SelectionAdapter fMenuItemSelectionListener = new SelectionAdapter() {
		public void widgetSelected(SelectionEvent e) {
			MenuItem item = (MenuItem) e.widget;
			if (item.getSelection()) {
				defaultRepo = (ITeamRepository) e.widget.getData();
				repositorySelected();
				openMetronomeWindow(defaultRepo
						.statistics(), defaultRepo.getRepositoryURI());
			}
		}
	};

	public MetronomeRepositorySelectionToolItem(ToolBar toolBar, int style,
			MetronomeModel metronomeModel) {
		this.metronomeModel = metronomeModel;
		mapMetronomeWindows = new HashMap<IStatistics, MetronomeWindow>();

		defaultRepo = restoreLastSelection();
		fItem = new ToolItem(toolBar, style | SWT.DROP_DOWN);
		fItem.addListener(SWT.Selection, fPopupListener);
		fItem.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				Iterator<MetronomeWindow> it = mapMetronomeWindows.values().iterator();
				while (it.hasNext()) {
					MetronomeWindow metronomeWindow = it
							.next();
					metronomeWindow.close();
				}
				if (fMenu != null)
					fMenu.dispose();
			}
		});
	}

	public ToolItem getControl() {
		return fItem;
	}

	/**
	 * Subclasses may override.
	 */
	protected void repositorySelected() {
		// TODO
	}

	private void createMenu() {
		checkSelectedRepository();
		if (fMenu != null)
			fMenu.dispose();
		fMenu = new Menu(fItem.getParent().getShell(), SWT.POP_UP);
		// not sending remove events since no repository will be removed while
		// the dialog is open.
		
		// Add the repositories to the menu
		StandardLabelProvider labelProvider = new StandardLabelProvider(
				new ElementRemovedNotifierImpl());
		ITeamRepository[] repos = TeamPlatform.getTeamRepositoryService().getTeamRepositories();
		for (int i = 0; i < repos.length; i++) {
			ITeamRepository repository = repos[i];
			MenuItem item = new MenuItem(fMenu, SWT.RADIO);
			String label = labelProvider.getText(repository);
			if (label == null)
				label = repository.getName();
			if (label == null)
				label = repository.getRepositoryURI();
			item.setText(label);
			item.setImage(labelProvider.getImage(repository));
			if (defaultRepo == repository)
				item.setSelection(true);
			// item.addListener(SWT.Selection,
			// fMenuItemSelectionListener);//TODO
			item.addSelectionListener(fMenuItemSelectionListener);
			item.setData(repository);
		}

		
		// Show a "Close" item
		{
			if (repos.length > 0) {
				new MenuItem(fMenu, SWT.SEPARATOR);
			}
			
			MenuItem close = new MenuItem(fMenu, SWT.PUSH);
			
			close.setText(Messages.MetronomeRepositorySelectionToolItem_closeAction);
			
			close.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					new InstanceScope().getNode(UiPlugin.PLUGIN_ID).putBoolean(UiPlugin.SHOW_METRONOME_TRIM, false);
				}
			});
		}
	}

	private void showPopupMenu(Event event) {
		createMenu();
		Rectangle bounds = fItem.getParent().getBounds();
		Point point = new Point(event.x, bounds.y + bounds.height);
		point = fItem.getDisplay().map(fItem.getParent(), null, point);
		fMenu.setLocation(point);
		fMenu.setVisible(true);
	}

	private ITeamRepository restoreLastSelection() {
		return getFirstActiveRepository();
	}

	private ITeamRepository getFirstActiveRepository() {
		ITeamRepository[] repos = TeamPlatform.getTeamRepositoryService().getTeamRepositories();
		for (int i = 0; i < repos.length; i++) {
			if (repos[i].loggedIn())
				return repos[i];
		}
		return null;
	}

	private void checkSelectedRepository() {
		if (defaultRepo == null || !defaultRepo.loggedIn())
			defaultRepo = getFirstActiveRepository();
	}

	public void openMetronomeWindow(IStatistics statistics, String repoName) {
		MetronomeWindow metronomeWindow;
		if (!mapMetronomeWindows.containsKey(statistics)
				|| mapMetronomeWindows.get(statistics)
						.getShell() == null) {
			metronomeWindow = new MetronomeWindow(metronomeModel, statistics,
					repoName);
			mapMetronomeWindows.put(statistics, metronomeWindow);
			metronomeWindow.open();
		}
		metronomeWindow = mapMetronomeWindows.get(statistics);
		metronomeWindow.setActive();
	}
}
