/*******************************************************************************
 * Licensed Materials - Property of IBM
 * © Copyright IBM Corporation 2005, 2015. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.metronome.internal;

import java.lang.ref.WeakReference;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.security.SecureRandom;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.IdentityHashMap;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.expressions.EvaluationResult;
import org.eclipse.core.expressions.ExpressionInfo;
import org.eclipse.core.expressions.IEvaluationContext;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.StatusLineManager;
import org.eclipse.jface.commands.ActionHandler;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.FontRegistry;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.window.ApplicationWindow;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTError;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.graphics.Rectangle;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.swt.widgets.TreeItem;
import org.eclipse.ui.ISources;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.contexts.IContextService;
import org.eclipse.ui.handlers.IHandlerActivation;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.internal.misc.StatusUtil;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.eclipse.ui.themes.ITheme;

import com.ibm.team.filesystem.client.internal.LoggingHelper;
import com.ibm.team.filesystem.client.internal.MetronomeModel;
import com.ibm.team.jface.itemview.MarkupUtil;
import com.ibm.team.metronome.MetronomePlugin;
import com.ibm.team.repository.client.IStatistics;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.client.ITeamRepositoryService.IRepositoryServiceListener;
import com.ibm.team.repository.client.TeamPlatform;
import com.ibm.team.repository.client.internal.TeamRepository;
import com.ibm.team.repository.client.internal.TeamRepositoryService;
import com.ibm.team.repository.client.util.IClientLibraryContext;
import com.ibm.team.repository.client.util.IClientLibraryContext.IServiceRunnable;
import com.ibm.team.repository.client.util.IListener;
import com.ibm.team.repository.client.util.PropertyChangeEvent;
import com.ibm.team.repository.common.IItemType;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.scm.admin.common.ISCMAdminService;

public class MetronomeWindow extends ApplicationWindow implements IListener, IRepositoryServiceListener {

	private MetronomeModel model;

	private IStatistics statistics;

	private String repoName;

	private Runnable runnable;
	
	private volatile Job job;

	private volatile boolean servicesStatsChanged;

	private volatile boolean itemsStatsChanged;
	
	private volatile long lastUpdate;

	private Image windowImage;

	private Clipboard cb;

	private static final int SVC_METHOD_COL_NUM = 0;

	private static final int SVC_CALL_COUNT_COL_NUM = 1;

	private static final int SVC_ELAPSED_TIME_COL_NUM = 2;

	private static final int SVC_TIME_RATIO_COL_NUM = 3;

	private static final int SVC_AVERAGE_TIME_COL_NUM = 4;

	private static final int SVC_WORST_TIME_COL_NUM = 5;

	private static final int SVC_TOOLTIP_COL_NUM = 6;

	private static final int ITEM_TYPE_COL_NUM = 0;

	private static final int ITEM_COUNT_COL_NUM = 1;

	private static final int ITEM_SIZE_COL_NUM = 2;

	private static final int ITEM_SIZE_RATIO_COL_NUM = 3;

	private static final int ITEM_HITS_COL_NUM = 4;

	private static final int ITEM_MISSES_COL_NUM = 5;

	private static final int ITEM_REFRESHES_COL_NUM = 6;

	private static final int STRING = 0;

	private static final int LONG = 1;

	private static final int DOUBLE = 2;

	private TabFolder tabFolder;

	private Tree servicesTree;

	private TreeColumn serviceMethodTreeColumn;

	private TreeColumn svcCallCountTreeColumn;

	private TreeColumn svcTimeTreeColumn;

	private TreeColumn svcTimeRatioTreeColumn;

	private TreeColumn srcAvgTimeTreeColumn;

	private TreeColumn srcWorstTimeTreeColumn;

	
	private Table repositoryTable;
	
	private TableColumn repositoryNameColumn;
    private TableColumn repositoryUriColumn;
    private TableColumn repositoryIdColumn;
    private TableColumn repositoryStateColumn;   
    private TableColumn repositoryCachedColumn;
    private TableColumn repositorySharedColumn;   

    private TableColumn[] repositoryTableColumns;
	
    
	private Table itemsTable;

	private TableColumn itemTypeTableColumn;

	private TableColumn itemCountTableColumn;

	private TableColumn itemSizeTableColumn;

	private TableColumn itemSizeRatioTableColumn;

	private TableColumn itemHitsTableColumn;

	private TableColumn itemMissesTableColumn;

	private TableColumn itemRefreshesTableColumn;

	private int selectedTabIndex;

	private IAction copyAction;

	private IAction closeAction;

	private IAction expandAllAction;

	private IAction collapseAllAction;

	private IAction resetAction;

	private Button copyServicesButton;

	private Button expandAllButton;

	private Button collapseAllButton;

	private Button resetServicesButton;

	private Button copyItemsButton;

	private Button resetItemsButton;

	private IHandlerActivation copyHandler;

	private Button runTestButton;

	private Label pingTimeValue;

	private Label upBandwidthValue;

	private Label downBandwidthValue;

	static class NetworkConnectionResults {
		public long pingTime = 0;
		public long pingTries = 0;
		public long pingMin = 0;
		public long pingMax = 0;
		public double upSpeed = 0;
		public double downSpeed = 0;
		public long pingStddev = 0;
	}
	
	 /**
     * Convert the given byte count to a human readable string.
     */
    final static double statisticalDivisors[] = { 1024.0, 1048576.0,
            1073741824.0, 1099511627776.0, 1125899906842624.0 };

    final static String statisticalSuffixes[] = { 
        "KB", //$NON-NLS-1$
        "MB", //$NON-NLS-1$
        "GB", //$NON-NLS-1$
        "TB", //$NON-NLS-1$
        "PB" //$NON-NLS-1$
    };
    
    private Color[] rangeColors = new Color[6];   

    public static String bytesAsHumanReadable(long amt) {

        for (int i = statisticalDivisors.length - 1; i >= 0; i--) {

            if (amt > statisticalDivisors[i]) {
                return String.format("%,.1f %s", amt / statisticalDivisors[i], //$NON-NLS-1$
                        statisticalSuffixes[i]);
            }

        }
        return NLS.bind("{0} B", amt); //$NON-NLS-1$
    }
    
	/**
	 * Create the application window
	 */
	public MetronomeWindow(MetronomeModel model, IStatistics statistics,
			String repoName) {
		super(null);
		addStatusLine();
		addMenuBar();
		this.model = model;
		this.statistics = statistics;
		this.repoName = repoName;
	}

	void setActive() {
		getShell().setActive();
	}

	/**
	 * Create contents of the application window
	 * 
	 * @param parent
	 */
	protected Control createContents(Composite parent) {
		if (model != null) {
			model.addGenericListener(IStatistics.SERVICE_METHOD, this);
			model.addGenericListener(IStatistics.ITEM_TYPE, this);
		}
		
		TeamPlatform.getTeamRepositoryService().addRepositoryServiceListener(this);
		for (ITeamRepository repo : getAllRepos()) {
		    repo.addGenericListener(repo, metronomeRepoListener);
		}
		
		rangeColors[0] = new Color(parent.getDisplay(), 51, 255, 51);
		rangeColors[1] = new Color(parent.getDisplay(), 170, 255, 170);
		rangeColors[2] = new Color(parent.getDisplay(), 204, 204, 255);
		rangeColors[3] = new Color(parent.getDisplay(), 255, 255, 153);
		rangeColors[4] = new Color(parent.getDisplay(), 255, 153, 153);
		rangeColors[5] = new Color(parent.getDisplay(), 255, 0, 0);

		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new FillLayout());

		tabFolder = new TabFolder(container, SWT.NONE);

		createServicesTab(tabFolder);
		createItemManagerTab(tabFolder);
		createNetworkConnectionTab(tabFolder);
		createRepositoryTab(tabFolder);
		
		tabFolder.addListener(SWT.Activate, new Listener() {
			public void handleEvent(Event e) {
				selectedTabIndex = tabFolder.getSelectionIndex();
				setEnabledForSelectedTab();
			}
		});

		tabFolder.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				selectedTabIndex = tabFolder.getSelectionIndex();
				setEnabledForSelectedTab();
			}
		});

		makeActions();

		servicesStatsChanged = true;
		itemsStatsChanged = true;
		
		runnable = new Runnable() {
			public void run() {
				updateView();
			}
		};
		
		job = new Job("") { //$NON-NLS-1$
			Display d = getShell().getDisplay();
			
			@Override
			protected IStatus run(IProgressMonitor monitor) {
				d.asyncExec(runnable);
				return Status.OK_STATUS;
			}
		};
		job.setUser(false);

		updateView();

		return container;
	}

	private Collection<ITeamRepository> getAllRepos() {
	    HashSet<ITeamRepository> repos = new HashSet<ITeamRepository>();
	    
	    for (ITeamRepository repo : TeamPlatform.getTeamRepositoryService().getTeamRepositories()) {
	        repos.add(repo);
	    }
	    
	    for (ITeamRepository repo : getRepositoryWeakReferences()) {
	        repos.add(repo);
	    }
	    
	    return repos;
	}

	private void createRepositoryTab(TabFolder tabFolder2) {
		final TabItem itemsTabItem = new TabItem(tabFolder, SWT.NONE);
		itemsTabItem.setText("Repositories"); //$NON-NLS-1$

		final Composite itemsComposite = new Composite(tabFolder2, SWT.NONE);
        itemsComposite.setLayout(new GridLayout());
        itemsComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		itemsTabItem.setControl(itemsComposite);

		repositoryTable = new Table(itemsComposite, SWT.BORDER | SWT.MULTI
				| SWT.FULL_SELECTION);
        repositoryTable.setLinesVisible(true);
        repositoryTable.setHeaderVisible(true);

        repositoryTable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		repositoryNameColumn = new TableColumn(repositoryTable, SWT.NONE);
		repositoryNameColumn.setText(Messages.MetronomeWindow_REPO_NAME); 
		repositoryNameColumn.setWidth(100);
		
        repositoryUriColumn = new TableColumn(repositoryTable, SWT.NONE);
        repositoryUriColumn.setText(Messages.MetronomeWindow_REPO_URI); 

        repositoryIdColumn = new TableColumn(repositoryTable, SWT.NONE);
        repositoryIdColumn.setText(Messages.MetronomeWindow_REPO_ID); 
        repositoryIdColumn.setWidth(100);

        repositoryStateColumn = new TableColumn(repositoryTable, SWT.NONE);
        repositoryStateColumn.setText(Messages.MetronomeWindow_REPO_STATE); 
        repositoryStateColumn.setWidth(100);
        
        repositoryCachedColumn = new TableColumn(repositoryTable, SWT.NONE);
        repositoryCachedColumn.setText(Messages.MetronomeWindow_REPO_IS_CACHED); 
        
        repositorySharedColumn = new TableColumn(repositoryTable, SWT.NONE);
        repositorySharedColumn.setText(Messages.MetronomeWindow_REPO_IS_SHARED); 

        repositoryTableColumns = new TableColumn[] {
        		repositoryNameColumn, 
        		repositoryUriColumn,
        		repositoryIdColumn,
        		repositoryStateColumn, 
        		repositoryCachedColumn,
        		repositorySharedColumn
        };
        
        updateRepositoryTab();
        
        Button refreshButton = new Button(itemsComposite, SWT.PUSH);
        refreshButton.setText("&Refresh"); //$NON-NLS-1$
        refreshButton.setLayoutData(new GridData(SWT.END, SWT.END, false, false));
        
        refreshButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				updateRepositoryTab();
			}
        });
	}

	private void updateRepositoryTab() {
        if (repositoryTable.isDisposed()) {
            return;
        }

        repositoryTable.removeAll();
        
        HashSet<ITeamRepository> weakRefs = getRepositoryWeakReferences();
        ITeamRepository[] cachedRepos = TeamPlatform.getTeamRepositoryService().getTeamRepositories();
        for (ITeamRepository repo : cachedRepos) {
            boolean wasCached = weakRefs.remove(repo);
            
            addRepositoryItem(repo, wasCached, true);
        }

        for (ITeamRepository repo : weakRefs) {
            addRepositoryItem(repo, true, false);
        }        
        
        repositoryTable.layout();
	}

	private HashSet<ITeamRepository> getRepositoryWeakReferences() {
	    Object object = null;
	    Exception ex = null;
		try {
			Field sr = TeamRepositoryService.class.getDeclaredField("sharedRepositories"); //$NON-NLS-1$
			sr.setAccessible(true);
			object = sr.get(TeamPlatform.getTeamRepositoryService());
		} catch (SecurityException e) {
		    ex = e;
		} catch (IllegalArgumentException e) {
		    ex = e;
		} catch (NoSuchFieldException e) {
		    ex = e;
		} catch (IllegalAccessException e) {
		    ex = e;
		}
		
		if (ex != null) {
			LoggingHelper.log(new Status(IStatus.ERROR, MetronomePlugin.PLUGIN_ID, "Unable to get sharedRepositories field", ex)); //$NON-NLS-1$
			return new HashSet<ITeamRepository>();
		}
		
		@SuppressWarnings("unchecked")
		Collection<WeakReference<ITeamRepository>> refs = (Collection<WeakReference<ITeamRepository>>) object;
	    
	    HashSet<ITeamRepository> repos = new HashSet<ITeamRepository>(refs.size());
	    for (WeakReference<ITeamRepository> ref : refs) {
	        ITeamRepository repo = ref.get();
	        if (repo != null) {
	            repos.add(repo);
	        }
	    }
	    
	    return repos;
	}
	
	private void addRepositoryItem(ITeamRepository repo, boolean wasCached, boolean wasShared) {
        TableItem item = new TableItem(repositoryTable, SWT.NONE);
        
        String idFromId = repo.getId() == null ? "null" : repo.getId().getUuidValue(); //$NON-NLS-1$
        String idFromRoot = repo.root() == null ? "null" : repo.root().getItemId().getUuidValue(); //$NON-NLS-1$
        
        String values[] = new String[] {
                repo.getName() == null ? repo.getRepositoryURI() : repo.getName(), 
                getUriString(repo), 
                idFromId + " (getId())\n" + idFromRoot + " (root().getItemId())", //$NON-NLS-1$ //$NON-NLS-2$
                getRepositoryState(repo),
                wasCached ? "Cached" : "", //$NON-NLS-1$ //$NON-NLS-2$
                wasShared ? "Shared" : ""  //$NON-NLS-1$//$NON-NLS-2$
        };
        
        item.setText(values);
        
        GC gc = new GC(repositoryTable.getDisplay());
        try {
	        for (int i = 0; i < values.length; i++) {
	        	int size;
	        	
	        	if (i == 1 || i == 2) {
	        		size = findMaxExtent(gc, values[i]);
	        	}
	        	else {
	        		size = gc.stringExtent(values[i]).x;
	        	}
	        	
	        	size += 10;
	        	
	        	if (size > repositoryTableColumns[i].getWidth()) {
	        		repositoryTableColumns[i].setWidth(size);
	        	}
	        }
        }
        finally {
        	gc.dispose();
        }
	}
	
	private int findMaxExtent(GC gc, String s) {
		String[] lines = s.split("\n"); //$NON-NLS-1$
		
		int max = Integer.MIN_VALUE;
		
		for (String line : lines) {
			max = Math.max(max, gc.stringExtent(line).x);
		}
		
		return max;
	}
	
	private String getRepositoryState(ITeamRepository repo) {
	    switch (repo.getErrorState()) {
	    case (ITeamRepository.ERROR_STATE_IO):
	        return "IO Error"; //$NON-NLS-1$
	    
	    case (ITeamRepository.ERROR_STATE_AUTH):
	        return "Auth Error"; //$NON-NLS-1$
	    
	    case (ITeamRepository.ERROR_STATE_NONE):
	        switch (repo.getState()) {
            case (ITeamRepository.STATE_LOGGING_IN):
                return "Logging In"; //$NON-NLS-1$
            
	        case (ITeamRepository.STATE_LOGGED_IN):
	            return "Logged In"; //$NON-NLS-1$
	        
            case (ITeamRepository.STATE_LOGGING_OUT):
                return "Logging Out"; //$NON-NLS-1$

            case (ITeamRepository.STATE_LOGGED_OUT):
                return "Logged Out"; //$NON-NLS-1$
            
            default:
                return "getState() == " + repo.getState(); //$NON-NLS-1$
	        }
	    
        default:
            return "getState() == " + repo.getErrorState();  //$NON-NLS-1$
	    }
	}

	private String getUriString(ITeamRepository repo) {
	    StringBuffer buf = new StringBuffer();
	    
	    buf.append(repo.getRepositoryURI());
	    buf.append(" (getRepositoryURI())\n"); //$NON-NLS-1$
	    buf.append(repo.publicUriRoot());
	    buf.append(" (publicUriRoot())"); //$NON-NLS-1$

	    for (Object uri : repo.getAliasRepositoryURIs()) {
            buf.append("\n"); //$NON-NLS-1$
	        buf.append(uri);
	        buf.append(" (alias)"); //$NON-NLS-1$
	    }
	    
	    return buf.toString();
	}

	private void createNetworkConnectionTab(final TabFolder tabFolder) {
		final TabItem networkConnectionTab = new TabItem(tabFolder, SWT.NONE);
		networkConnectionTab.setText(Messages.MetronomeWindow_networkConnectionTab);

		GridLayout gridLayout = new GridLayout();
		final Composite networkConnectionComposite = new Composite(tabFolder, SWT.NONE);
		gridLayout = new GridLayout();
		gridLayout.numColumns = 2;
		networkConnectionComposite.setLayout(gridLayout);
		networkConnectionTab.setControl(networkConnectionComposite);
		
		runTestButton = new Button(networkConnectionComposite, SWT.PUSH);
		runTestButton.setText(Messages.MetronomeWindow_testSpeedButton);
		runTestButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				// this will block and return when the test is done
				runConnectionTest();				
			}
		});
		
		GridDataFactory.fillDefaults().span(2, 1).applyTo(runTestButton);
		
		Label pingTimeLabel = new Label(networkConnectionComposite, SWT.WRAP);
		pingTimeLabel.setText(Messages.MetronomeWindow_pingTime);
				
		pingTimeValue = new Label(networkConnectionComposite, SWT.WRAP);
		pingTimeValue.setText(Messages.MetronomeWindow_pingTimeDefault);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(pingTimeValue);
		
		Label upBandwidthLabel = new Label(networkConnectionComposite, SWT.WRAP);
		upBandwidthLabel.setText(Messages.MetronomeWindow_uploadSpeed);
		
		upBandwidthValue = new Label(networkConnectionComposite, SWT.WRAP);
		upBandwidthValue.setText(Messages.MetronomeWindow_uploadSpeedDefault);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(upBandwidthValue);
		
		Label downBandwidthLabel = new Label(networkConnectionComposite, SWT.WRAP);
		downBandwidthLabel.setText(Messages.MetronomeWindow_downloadSpeed);
		
		downBandwidthValue = new Label(networkConnectionComposite, SWT.WRAP);
		downBandwidthValue.setText(Messages.MetronomeWindow_downloadSpeedDefault);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(downBandwidthValue);
		
		Color backgroundColor = downBandwidthValue.getDisplay().getSystemColor(
                SWT.COLOR_WIDGET_BACKGROUND);
		
		// Allow the browser to fail out.
		try {
			Browser browser = new Browser(networkConnectionComposite, SWT.NONE); 
			
			String text = NLS.bind(Messages.MetronomeWindow_explanationHtml,
			   new String[] {
				   getCssIntro(), 
				   Integer.toHexString(backgroundColor.getRGB().red) + Integer.toHexString(backgroundColor.getRGB().green) + Integer.toHexString(backgroundColor.getRGB().blue),
				   statistics.teamRepository().getRepositoryURI()
				});
			
			browser.setText(text);
			
			browser.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
		} catch (SWTError e) {
			// Ignore e
			IStatus status = new Status(IStatus.INFO, MetronomePlugin.PLUGIN_ID, "Missing browser library", e); //$NON-NLS-1$
			LoggingHelper.log(status);
		}
	}
	
	private String getCssIntro() {
		FontData font= JFaceResources.getDefaultFont().getFontData()[0];

		StringBuffer css= new StringBuffer();

		css.append("<style type=\"text/css\">\n"); //$NON-NLS-1$
		css.append("body { overflow: auto; ").append(MarkupUtil.getFont(font.getName(), font.getHeight())).append(" }\n"); //$NON-NLS-1$ //$NON-NLS-2$
		css.append("th { ").append(MarkupUtil.getFont(font.getName(), font.getHeight())).append(" }\n"); //$NON-NLS-1$ //$NON-NLS-2$
		css.append("tr {  ").append(MarkupUtil.getFont(font.getName(), font.getHeight())).append(" }\n"); //$NON-NLS-1$ //$NON-NLS-2$
		css.append("span.photo { padding: 3 10 0 3; float: left; }\n"); //$NON-NLS-1$

		css.append("span.name { padding: 5 0 0 0; float: left; "); //$NON-NLS-1$
		css.append(MarkupUtil.getFont(font.getName(), font.getHeight() + 2, SWT.BOLD));
		css.append("}\n"); //$NON-NLS-1$

		css.append("a:link { text-decoration: none; }\n"); //$NON-NLS-1$
		css.append("a:visited { color:#0000FF; text-decoration: none; }\n"); //$NON-NLS-1$

		css.append("a:hover { text-decoration: underline; }\n"); //$NON-NLS-1$

		css.append("</style>\n"); //$NON-NLS-1$
		return css.toString();
	}
	
	private void createServicesTab(final TabFolder tabFolder) {
		final TabItem servicesTabItem = new TabItem(tabFolder, SWT.NONE);
		servicesTabItem.setText(Messages.MetronomeWindow_servicesTab);

		final Composite servicesComposite = new Composite(tabFolder, SWT.NONE);
		servicesComposite.setLayout(new GridLayout());
		servicesTabItem.setControl(servicesComposite);

		servicesTree = new Tree(servicesComposite, SWT.BORDER | SWT.MULTI
				| SWT.FULL_SELECTION);
		servicesTree.setLinesVisible(true);
		servicesTree.setHeaderVisible(true);
		servicesTree
				.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		serviceMethodTreeColumn = new TreeColumn(servicesTree, SWT.NONE);
		serviceMethodTreeColumn.setText(Messages.MetronomeWindow_serviceMethodColumnHeading);

		svcCallCountTreeColumn = new TreeColumn(servicesTree, SWT.NONE);
		svcCallCountTreeColumn.setText(Messages.MetronomeWindow_countColumnHeading);
		svcCallCountTreeColumn.setAlignment(SWT.RIGHT);

		svcTimeTreeColumn = new TreeColumn(servicesTree, SWT.NONE);
		svcTimeTreeColumn.setText(Messages.MetronomeWindow_timeColumnHeading);
		svcTimeTreeColumn.setAlignment(SWT.RIGHT);

		svcTimeRatioTreeColumn = new TreeColumn(servicesTree, SWT.NONE);
		svcTimeRatioTreeColumn.setText(Messages.MetronomeWindow_timePercentColumnHeading);
		svcTimeRatioTreeColumn.setAlignment(SWT.RIGHT);

		srcAvgTimeTreeColumn = new TreeColumn(servicesTree, SWT.NONE);
		srcAvgTimeTreeColumn.setText(Messages.MetronomeWindow_averageSecondsColumnHeading);
		srcAvgTimeTreeColumn.setAlignment(SWT.RIGHT);

		srcWorstTimeTreeColumn = new TreeColumn(servicesTree, SWT.NONE);
		srcWorstTimeTreeColumn.setText(Messages.MetronomeWindow_worstSecondsColumnHeading);
		srcWorstTimeTreeColumn.setAlignment(SWT.RIGHT);

		// NOTE: only to store tooltip text so it is hidden
		TreeColumn tooltipTreeColumn = new TreeColumn(servicesTree, SWT.NONE);
		tooltipTreeColumn.setWidth(0);
		tooltipTreeColumn.setResizable(false);

		createFullNameTooltip();

		Listener sortListener = new Listener() {
			public void handleEvent(Event e) {
				sortServicesTable(e);
			}
		};
		serviceMethodTreeColumn.addListener(SWT.Selection, sortListener);
		svcCallCountTreeColumn.addListener(SWT.Selection, sortListener);
		svcTimeTreeColumn.addListener(SWT.Selection, sortListener);
		svcTimeRatioTreeColumn.addListener(SWT.Selection, sortListener);
		srcAvgTimeTreeColumn.addListener(SWT.Selection, sortListener);
		srcWorstTimeTreeColumn.addListener(SWT.Selection, sortListener);

		final Composite buttonsComposite = new Composite(servicesComposite,
				SWT.NONE);
		GridLayout layout = new GridLayout(5, false);
		buttonsComposite.setLayout(layout);
		buttonsComposite.setLayoutData(new GridData(SWT.END, SWT.CENTER, false,
				false));

		Label label = new Label(buttonsComposite, SWT.NONE);
		label.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false));
		label.setVisible(false);

		copyServicesButton = new Button(buttonsComposite, SWT.NONE);
		copyServicesButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER,
				false, false, 1, 1));
		copyServicesButton.setText(Messages.MetronomeWindow_copyButton);
		copyServicesButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				copyToClipboard();
			}
		});

		servicesTree.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				setEnabledForCopyServices();
			}
		});

		expandAllButton = new Button(buttonsComposite, SWT.NONE);
		expandAllButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER,
				false, false, 1, 1));
		expandAllButton.setText(Messages.MetronomeWindow_expandAllButton);
		expandAllButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				expandServicesTree();
			}
		});

		collapseAllButton = new Button(buttonsComposite, SWT.NONE);
		collapseAllButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER,
				false, false, 1, 1));
		collapseAllButton.setText(Messages.MetronomeWindow_collapseAllButton);
		collapseAllButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				collapseServicesTree();
			}
		});

		resetServicesButton = new Button(buttonsComposite, SWT.NONE);
		resetServicesButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER,
				false, false, 1, 1));
		resetServicesButton.setText(Messages.MetronomeWindow_resetButton);
		resetServicesButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				resetServicesStats();
			}
		});

		servicesComposite.addControlListener(new ControlAdapter() {
			public void controlResized(ControlEvent e) {
				resizeServicesTree(servicesComposite);
			}
		});

		updateServiceCountTab();
	}

	private void createFullNameTooltip() {
		// Disable native tooltip
		servicesTree.setToolTipText(""); //$NON-NLS-1$

		// Implement a "fake" tooltip
		final Listener labelListener = new Listener() {
			public void handleEvent(Event event) {
				Label label = (Label) event.widget;
				Shell shell = label.getShell();
				switch (event.type) {
				case SWT.MouseDown:
					Event e = new Event();
					e.item = (TreeItem) label.getData("_TABLEITEM"); //$NON-NLS-1$
					// Assuming table is single select, set the selection as if
					// the mouse down event went through to the table
					servicesTree
							.setSelection(new TreeItem[] { (TreeItem) e.item });
					servicesTree.notifyListeners(SWT.Selection, e);
					// fall through
				case SWT.MouseExit:
					shell.dispose();
					break;
				}
			}
		};

		Listener tableListener = new Listener() {
			Shell tip = null;

			Label label = null;

			public void handleEvent(Event event) {
				switch (event.type) {
				case SWT.Dispose:
				case SWT.KeyDown:
				case SWT.MouseMove: {
					if (tip == null)
						break;
					tip.dispose();
					tip = null;
					label = null;
					break;
				}
				case SWT.MouseHover: {
					Point ptOfMouseHover = new Point(event.x, event.y);
					TreeItem item = servicesTree.getItem(ptOfMouseHover);
					if (item != null) {
						Rectangle rect = item.getBounds(SVC_METHOD_COL_NUM);
						if (rect.contains(ptOfMouseHover)) {
							if (tip != null && !tip.isDisposed())
								tip.dispose();
							tip = new Shell(getShell(), SWT.ON_TOP | SWT.TOOL);
							tip.setLayout(new FillLayout());
							label = new Label(tip, SWT.NONE);
							label.setForeground(getShell().getDisplay()
									.getSystemColor(SWT.COLOR_INFO_FOREGROUND));
							label.setBackground(getShell().getDisplay()
									.getSystemColor(SWT.COLOR_INFO_BACKGROUND));
							label.setData("_TABLEITEM", item); //$NON-NLS-1$
							// text of hidden column becomes tool tip
							label.setText(item.getText(SVC_TOOLTIP_COL_NUM));
							label.addListener(SWT.MouseExit, labelListener);
							label.addListener(SWT.MouseDown, labelListener);
							Point size = tip.computeSize(SWT.DEFAULT,
									SWT.DEFAULT);
							Point ptToDisplay = servicesTree.toDisplay(rect.x,
									rect.y);
							tip.setBounds(ptToDisplay.x, ptToDisplay.y, size.x,
									size.y);
							tip.setVisible(true);
						}
					}
				}
				}
			}
		};
		servicesTree.addListener(SWT.Dispose, tableListener);
		servicesTree.addListener(SWT.KeyDown, tableListener);
		servicesTree.addListener(SWT.MouseMove, tableListener);
		servicesTree.addListener(SWT.MouseHover, tableListener);
	}

	private void createItemManagerTab(TabFolder tabFolder) {
		final TabItem itemsTabItem = new TabItem(tabFolder, SWT.NONE);
		itemsTabItem.setText(Messages.MetronomeWindow_itemManagerTab);

		final Composite itemsComposite = new Composite(tabFolder, SWT.NONE);
		itemsComposite.setLayout(new GridLayout());
		itemsTabItem.setControl(itemsComposite);

		itemsTable = new Table(itemsComposite, SWT.BORDER | SWT.MULTI
				| SWT.FULL_SELECTION);
		itemsTable.setLinesVisible(true);
		itemsTable.setHeaderVisible(true);
		itemsTable.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));

		itemTypeTableColumn = new TableColumn(itemsTable, SWT.NONE);
		itemTypeTableColumn.setText(Messages.MetronomeWindow_itemTypeColumnHeading);

		itemCountTableColumn = new TableColumn(itemsTable, SWT.NONE);
		itemCountTableColumn.setText(Messages.MetronomeWindow_itemCountColumnHeading);
		itemCountTableColumn.setAlignment(SWT.RIGHT);

		itemSizeTableColumn = new TableColumn(itemsTable, SWT.NONE);
		itemSizeTableColumn.setText(Messages.MetronomeWindow_sizeBytesColumnHeading);
		itemSizeTableColumn.setAlignment(SWT.RIGHT);

		itemSizeRatioTableColumn = new TableColumn(itemsTable, SWT.NONE);
		itemSizeRatioTableColumn.setText(Messages.MetronomeWindow_sizePercentColumnHeading);
		itemSizeRatioTableColumn.setAlignment(SWT.RIGHT);

		itemHitsTableColumn = new TableColumn(itemsTable, SWT.NONE);
		itemHitsTableColumn.setText(Messages.MetronomeWindow_hitCountColumnHeading);
		itemHitsTableColumn.setAlignment(SWT.RIGHT);

		itemMissesTableColumn = new TableColumn(itemsTable, SWT.NONE);
		itemMissesTableColumn.setText(Messages.MetronomeWindow_missCountColumnHeading);
		itemMissesTableColumn.setAlignment(SWT.RIGHT);

		itemRefreshesTableColumn = new TableColumn(itemsTable, SWT.NONE);
		itemRefreshesTableColumn.setText(Messages.MetronomeWindow_refreshCountColumnHeading);
		itemRefreshesTableColumn.setAlignment(SWT.RIGHT);

		Listener sortListener = new Listener() {
			public void handleEvent(Event e) {
				sortItemsTable(e);
			}
		};
		itemTypeTableColumn.addListener(SWT.Selection, sortListener);
		itemCountTableColumn.addListener(SWT.Selection, sortListener);
		itemSizeTableColumn.addListener(SWT.Selection, sortListener);
		itemSizeRatioTableColumn.addListener(SWT.Selection, sortListener);
		itemHitsTableColumn.addListener(SWT.Selection, sortListener);
		itemMissesTableColumn.addListener(SWT.Selection, sortListener);
		itemRefreshesTableColumn.addListener(SWT.Selection, sortListener);

		final Composite buttonsComposite = new Composite(itemsComposite,
				SWT.NONE);
		GridLayout layout = new GridLayout(3, false);
		buttonsComposite.setLayout(layout);
		buttonsComposite.setLayoutData(new GridData(SWT.END, SWT.CENTER, false,
				false));

		Label label = new Label(buttonsComposite, SWT.NONE);
		label.setLayoutData(new GridData(SWT.CENTER, SWT.CENTER, true, false));
		label.setVisible(false);

		copyItemsButton = new Button(buttonsComposite, SWT.NONE);
		copyItemsButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER,
				false, false, 1, 1));
		copyItemsButton.setText(Messages.MetronomeWindow_copyButton);
		copyItemsButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				copyToClipboard();
			}
		});

		itemsTable.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event e) {
				setEnabledForCopyItems();
			}
		});

		resetItemsButton = new Button(buttonsComposite, SWT.NONE);
		resetItemsButton.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER,
				false, false, 1, 1));
		resetItemsButton.setText(Messages.MetronomeWindow_resetButton);
		resetItemsButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				resetItemsStats();
			}
		});

		itemsComposite.addControlListener(new ControlAdapter() {
			public void controlResized(ControlEvent e) {
				resizeItemsTable(itemsComposite);
			}
		});

		updateItemManagerTab();
	}

	private static Comparator<TreeItem> getTreeItemComparator(int type, final int index, boolean up) {
		if (up) {
			if (type == STRING) {
				return new Comparator<TreeItem>() {
					public int compare(TreeItem o1, TreeItem o2) {
						return o1.getText(index).compareTo(o2.getText(index));
					}
				};
			}
			if (type == LONG) {
				return new Comparator<TreeItem>() {
					public int compare(TreeItem o1, TreeItem o2) {
						return Long.valueOf(o1.getText(index)).compareTo(Long.valueOf(o2.getText(index)));
					}
				};
			}
			if (type == DOUBLE) {
				return new Comparator<TreeItem>() {
					public int compare(TreeItem o1, TreeItem o2) {
						return Double.valueOf(o1.getText(index)).compareTo(Double.valueOf(o2.getText(index)));
					}
				};
			}
		} else {
			if (type == STRING) {
				return new Comparator<TreeItem>() {
					public int compare(TreeItem o1, TreeItem o2) {
						return o2.getText(index).compareTo(o1.getText(index));
					}
				};
			}
			if (type == LONG) {
				return new Comparator<TreeItem>() {
					public int compare(TreeItem o1, TreeItem o2) {
						return Long.valueOf(o2.getText(index)).compareTo(Long.valueOf(o1.getText(index)));
					}
				};
			}
			if (type == DOUBLE) {
				return new Comparator<TreeItem>() {
					public int compare(TreeItem o1, TreeItem o2) {
						return Double.valueOf(o2.getText(index)).compareTo(Double.valueOf(o1.getText(index)));
					}
				};
			}
		}
		throw new IllegalArgumentException();
	}
	
	private static Comparator<TableItem> getTableItemComparator(int type, final int index, boolean up) {
		if (up) {
			if (type == STRING) {
				return new Comparator<TableItem>() {
					public int compare(TableItem o1, TableItem o2) {
						return o1.getText(index).compareTo(o2.getText(index));
					}
				};
			}
			if (type == LONG) {
				return new Comparator<TableItem>() {
					public int compare(TableItem o1, TableItem o2) {
						return Long.valueOf(o1.getText(index)).compareTo(Long.valueOf(o2.getText(index)));
					}
				};
			}
			if (type == DOUBLE) {
				return new Comparator<TableItem>() {
					public int compare(TableItem o1, TableItem o2) {
						return Double.valueOf(o1.getText(index)).compareTo(Double.valueOf(o2.getText(index)));
					}
				};
			}
		} else {
			if (type == STRING) {
				return new Comparator<TableItem>() {
					public int compare(TableItem o1, TableItem o2) {
						return o2.getText(index).compareTo(o1.getText(index));
					}
				};
			}
			if (type == LONG) {
				return new Comparator<TableItem>() {
					public int compare(TableItem o1, TableItem o2) {
						return Long.valueOf(o2.getText(index)).compareTo(Long.valueOf(o1.getText(index)));
					}
				};
			}
			if (type == DOUBLE) {
				return new Comparator<TableItem>() {
					public int compare(TableItem o1, TableItem o2) {
						return Double.valueOf(o2.getText(index)).compareTo(Double.valueOf(o1.getText(index)));
					}
				};
			}
		}
		throw new IllegalArgumentException();
	}
	
	private void runConnectionTest() {
		final TeamRepository teamRepository = (TeamRepository)statistics.teamRepository();
		final ISCMAdminService adminService = (ISCMAdminService) (teamRepository).getServiceInterface(ISCMAdminService.class);		
		ProgressMonitorDialog pmd = new ProgressMonitorDialog(tabFolder.getShell());
		final NetworkConnectionResults results = new NetworkConnectionResults();
		try {
			pmd.run(true, true, new IRunnableWithProgress() {
				public void run(IProgressMonitor monitor) throws InvocationTargetException,
						InterruptedException {
					try {						
						IServiceRunnable< Object> serviceRunnable = new IClientLibraryContext.IServiceRunnable<Object>() {
							public Object run(IProgressMonitor monitor) throws TeamRepositoryException {
								monitor.setTaskName(Messages.MetronomeWindow_testingConnectionProgress);
								monitor.beginTask(Messages.MetronomeWindow_testingConnectionProgress, 100);
								int NUM_PING_TRIES = 20;
								int FILE_SIZE = 2048000;
								int FILE_TRIES = 1;
								List<Long> times = new ArrayList<Long>();
								long time= System.currentTimeMillis();
								monitor.subTask(Messages.MetronomeWindow_pingTestsProgress);
								for(int i = 0; i < NUM_PING_TRIES; i++) {
									long singleTime = System.currentTimeMillis();
									adminService.echo(null);									 
									times.add(System.currentTimeMillis() - singleTime);
								}
								monitor.worked(30);								
						      
 					           	int count = 0;
 					           	double sum = 0.0;
 					           	double sumSquare = 0.0;
 					           	for(Long n : times) {
					               count++;
					               sum += n.doubleValue();
					               sumSquare += n.doubleValue() * n.doubleValue();
 					           	}
 					           	double average = sum / count;
 					           	results.pingTime = (long) average;
 					           	results.pingStddev = (long) Math.sqrt((sumSquare / count) - (average * average));
								results.pingMax = Collections.max(times);
								results.pingMin = Collections.min(times);
								results.pingTries = count;
								
								monitor.subTask(NLS.bind(Messages.MetronomeWindow_sendingFileProgress, bytesAsHumanReadable(FILE_SIZE)));
								time= System.currentTimeMillis();
								for(int i = 0; i < FILE_TRIES; i++) {
									adminService.echoUp(new Object[] {makeBytesToSend(FILE_SIZE)});
								}
								monitor.worked(35);
								time = (System.currentTimeMillis() - time);
								results.upSpeed = ((FILE_TRIES * FILE_SIZE) / (time / 1000.0));
								
								monitor.subTask(NLS.bind(Messages.MetronomeWindow_downloadingFileProgress, bytesAsHumanReadable(FILE_SIZE)));
								time= System.currentTimeMillis();
								for(int i = 0; i < FILE_TRIES; i++) {
									adminService.echoDown(FILE_SIZE);
								}
								monitor.worked(35);
								time = (System.currentTimeMillis() - time);
								results.downSpeed = ((FILE_TRIES * FILE_SIZE) / (time / 1000.0));								
								return null;
							}
						};
						((TeamRepository)teamRepository).callCancelableService(serviceRunnable, monitor);
						
						tabFolder.getDisplay().asyncExec(new Runnable() {
							public void run() {
								if(! tabFolder.getShell().isDisposed()) {
									pingTimeValue.setText(NLS.bind(Messages.MetronomeWindow_pingTimeFormat,
											new String[] {
												""+ results.pingTries, //$NON-NLS-1$
												""+ results.pingMin, //$NON-NLS-1$
												""+ results.pingTime, //$NON-NLS-1$
												""+ results.pingMax, //$NON-NLS-1$
												""+ results.pingStddev} //$NON-NLS-1$
										));
									upBandwidthValue.setText(NLS.bind(Messages.MetronomeWindow_perSecond, bytesAsHumanReadable((long)results.upSpeed)));
									downBandwidthValue.setText(NLS.bind(Messages.MetronomeWindow_perSecond, bytesAsHumanReadable((long)results.downSpeed)));
								}
							}
						});						
					} catch (TeamRepositoryException e) {
						throw new InvocationTargetException(e, Messages.MetronomeWindow_errorContactingServer);
					}
				}
			});
		} catch (InvocationTargetException e) {
			ErrorDialog.openError(tabFolder.getShell(), Messages.MetronomeWindow_connectionErrorDialogTitle, Messages.MetronomeWindow_netConnectionErrorMessage, 
					StatusUtil.newStatus(IStatus.ERROR, Messages.MetronomeWindow_errorStatusMessage, e.getTargetException()));
		} catch (InterruptedException e) {
			//
		}		
	}
	
	private String makeBytesToSend(int size) {
		SecureRandom randon = new SecureRandom();
		randon.setSeed(System.currentTimeMillis());
		byte[] randomBytes = new byte[size];
		String string2 = "abcedfghijklmnopqrstuvwxyz"; //$NON-NLS-1$
		for(int i = 0; i < size; i++) {			
			randomBytes[i] = (byte) string2.charAt(randon.nextInt(string2.length()));
		}
		return new String(randomBytes);		
	}
	
	public boolean close() {
		windowImage.dispose();
		if (cb != null)
			cb.dispose();
		
		if (model != null) {
			model.removeGenericListener(IStatistics.SERVICE_METHOD, this);
			model.removeGenericListener(IStatistics.ITEM_TYPE, this);
		}
		
		TeamPlatform.getTeamRepositoryService().removeRepositoryServiceListener(this);
		for (ITeamRepository repo : getAllRepos()) {
			repo.removeGenericListener(repo, metronomeRepoListener);
		}


		for (int i = 0; i < rangeColors.length; i++) {
			rangeColors[i].dispose();
		}
		// for Ctrl+C to work
		IContextService contextService = (IContextService) PlatformUI
				.getWorkbench().getService(IContextService.class);
		contextService.unregisterShell(getShell());
		IHandlerService handlerService = (IHandlerService) PlatformUI
				.getWorkbench().getService(IHandlerService.class);
		handlerService.deactivateHandler(copyHandler);
		return super.close();
	}
	
	void updateView() {
		lastUpdate = System.currentTimeMillis();
		if (getShell() == null || getShell().isDisposed())
			return;
		if (servicesStatsChanged) {
			updateServiceCountTab();
		}
		if (itemsStatsChanged)
			updateItemManagerTab();
	}

	private String getStatus(IStatistics statistics) {
		String str = 
			NLS.bind(Messages.MetronomeWindow_0, new String[]{
				""+ model.getTotalMethodCount(statistics), //$NON-NLS-1$
				""+ model.getTotalElapsedTime(statistics), //$NON-NLS-1$
				""+ model.getTotalItemCount(statistics), //$NON-NLS-1$
				""+ model.getTotalCacheSize(statistics) //$NON-NLS-1$
			});
		return str;
	}

	/**
	 * Create the status line manager
	 * 
	 * @return the status line manager
	 */
	protected StatusLineManager createStatusLineManager() {
		StatusLineManager statusLineManager = new StatusLineManager();
		return statusLineManager;
	}

	private void makeActions() {
		copyAction = new Action(Messages.MetronomeWindow_copyAction) {
			public void run() {
				copyToClipboard();
			}
		};
		// for Ctrl+C to work
		copyAction.setActionDefinitionId("org.eclipse.ui.edit.copy"); //$NON-NLS-1$

		closeAction = new Action(Messages.MetronomeWindow_closeAction) {
			public void run() {
				close();
			}
		};

		expandAllAction = new Action(Messages.MetronomeWindow_expandAllAction) {
			public void run() {
				expandServicesTree();
			}
		};

		collapseAllAction = new Action(Messages.MetronomeWindow_collapseAllAction) {
			public void run() {
				collapseServicesTree();
			}
		};

		resetAction = new Action(Messages.MetronomeWindow_resetAction) {
			public void run() {
				if (selectedTabIndex == 0) {
					resetServicesStats();
				} else {
					resetItemsStats();
				}
			}
		};
	}

	protected MenuManager createMenuManager() {
		MenuManager mgr = super.createMenuManager();

		MenuManager fileMenu = new MenuManager(Messages.MetronomeWindow_fileMenuAction);
		MenuManager viewMenu = new MenuManager(Messages.MetronomeWindow_viewMenuAction);
		mgr.add(fileMenu);
		mgr.add(viewMenu);

		makeActions();

		fileMenu.add(copyAction);
		fileMenu.add(closeAction);
		viewMenu.add(expandAllAction);
		viewMenu.add(collapseAllAction);
		viewMenu.add(resetAction);
		return mgr;
	}

	/**
	 * Configure the shell
	 * 
	 * @param newShell
	 */
	protected void configureShell(Shell newShell) {
		super.configureShell(newShell);
		newShell.setText(NLS.bind(Messages.MetronomeWindow_metronomeDialogTitle, repoName));
		newShell.setSize(580, 320);
		windowImage = AbstractUIPlugin.imageDescriptorFromPlugin(
				MetronomePlugin.PLUGIN_ID, "icons/obj16/metronome.gif") //$NON-NLS-1$
				.createImage(true, newShell.getDisplay());
		newShell.setImage(windowImage);
	}

	@SuppressWarnings({"unchecked", "rawtypes"})
	public void handleEvents(List events) {
		boolean needUpdate = false;
		for (Iterator<PropertyChangeEvent> iEvents = events.iterator(); iEvents.hasNext();) {
			PropertyChangeEvent event = iEvents.next();
			if (event.getData().equals(IStatistics.SERVICE_METHOD)) {
				servicesStatsChanged = true;
				needUpdate = true;
			} else {
				itemsStatsChanged = true;
				needUpdate = true;
			}
		}
		if (needUpdate) {
			long time = Math.max(0, lastUpdate + 1000 - System.currentTimeMillis());
			Job j = job;
			if (j != null) {
				j.schedule(time);
			}
		}
	}

	private void sortServicesTable(Event e) {
		TreeColumn nextSortColumn;
		int nextSortDir = servicesTree.getSortDirection();
		if (e == null) {
			nextSortColumn = servicesTree.getSortColumn();
		} else {
			TreeColumn currentSortColumn = servicesTree.getSortColumn();
			nextSortColumn = (TreeColumn) e.widget;
			if (currentSortColumn == null
					|| currentSortColumn != nextSortColumn) {
				servicesTree.setSortColumn(nextSortColumn);
				nextSortDir = SWT.UP;
			} else {
				nextSortDir = nextSortDir == SWT.UP ? SWT.DOWN : SWT.UP;
			}
			servicesTree.setSortDirection(nextSortDir);
		}

		int index = SVC_METHOD_COL_NUM;
		int type = STRING;
		if (nextSortColumn == svcCallCountTreeColumn) {
			index = SVC_CALL_COUNT_COL_NUM;
			type = LONG;
		} else if (nextSortColumn == svcTimeTreeColumn) {
			index = SVC_ELAPSED_TIME_COL_NUM;
			type = DOUBLE;
		} else if (nextSortColumn == svcTimeRatioTreeColumn) {
			index = SVC_TIME_RATIO_COL_NUM;
			type = LONG;
		} else if (nextSortColumn == srcAvgTimeTreeColumn) {
			index = SVC_AVERAGE_TIME_COL_NUM;
			type = DOUBLE;
		} else if (nextSortColumn == srcWorstTimeTreeColumn) {
			index = SVC_WORST_TIME_COL_NUM;
			type = DOUBLE;
		}
		
		Comparator<TreeItem> comparator = getTreeItemComparator(type, index, nextSortDir == SWT.UP);

		TreeItem[] items = servicesTree.getItems();
		
		TreeItem[] selected = servicesTree.getSelection();
		IdentityHashMap<TreeItem, Object> selection = new IdentityHashMap<TreeItem, Object>(selected.length * 4 / 3);
		for (TreeItem item : selected) {
			selection.put(item, item);
		}
		selected = null;
		
		TreeItem[] itemsClone = items.clone();
		Arrays.sort(itemsClone, comparator);
		int itemsStart = itemsClone.length;
		for (int i = 0; i < itemsClone.length; i++) {
			TreeItem item = itemsClone[i];
			boolean diffFound = itemsStart != itemsClone.length;
			if (!diffFound && item != items[i]) {
				itemsStart = i;
				diffFound = true;
			}
			TreeItem newItem = null;
			boolean expanded = false;
			if (diffFound) {
				String[] values = {
						item.getText(SVC_METHOD_COL_NUM),
						item.getText(SVC_CALL_COUNT_COL_NUM),
						item.getText(SVC_ELAPSED_TIME_COL_NUM),
						item.getText(SVC_TIME_RATIO_COL_NUM),
						item.getText(SVC_AVERAGE_TIME_COL_NUM),
						item.getText(SVC_WORST_TIME_COL_NUM),
						item.getText(SVC_TOOLTIP_COL_NUM) };
				newItem = new TreeItem(servicesTree, SWT.NONE);
				newItem.setText(values);
				newItem.setFont(item.getFont());
				if (selection.remove(item) != null) {
					selection.put(newItem, null);
				}
				expanded = item.getExpanded();
			}
			
			TreeItem[] subItems = item.getItems();
			
			
			TreeItem[] clone = diffFound ? subItems : subItems.clone();
			
			
			Arrays.sort(clone, comparator);
			int start = 0;
			if (!diffFound) {
				for (; start < clone.length && clone[start] == subItems[start]; start ++);
			}
			subItems = null;
			
			for (int j = start; j < clone.length; j++) {
				String[] values = {
						clone[j].getText(SVC_METHOD_COL_NUM),
						clone[j].getText(SVC_CALL_COUNT_COL_NUM),
						clone[j].getText(SVC_ELAPSED_TIME_COL_NUM),
						clone[j].getText(SVC_TIME_RATIO_COL_NUM),
						clone[j].getText(SVC_AVERAGE_TIME_COL_NUM),
						clone[j].getText(SVC_WORST_TIME_COL_NUM),
						clone[j].getText(SVC_TOOLTIP_COL_NUM) };
				TreeItem subItem = new TreeItem(newItem == null ? item : newItem, SWT.NONE);
				subItem.setText(values);
				if (selection.remove(clone[j]) != null) {
					selection.put(subItem, null);
				}
			}
			for (int j = start; j < clone.length; j++) {
				clone[j].dispose();
			}
			if (newItem != null) {
				newItem.setExpanded(expanded);
			}
		}
		
		for (int i = itemsStart; i < itemsClone.length; i++) {
			items[i].dispose();
		}
		
		servicesTree.setSelection(selection.keySet().toArray(new TreeItem[selection.size()]));
	}

	private void sortItemsTable(Event e) {
		TableColumn nextSortColumn;
		int nextSortDir = itemsTable.getSortDirection();
		if (e == null) {
			nextSortColumn = itemsTable.getSortColumn();
		} else {
			TableColumn currentSortColumn = itemsTable.getSortColumn();
			nextSortColumn = (TableColumn) e.widget;
			if (currentSortColumn == null
					|| currentSortColumn != nextSortColumn) {
				itemsTable.setSortColumn(nextSortColumn);
				nextSortDir = SWT.UP;
			} else {
				nextSortDir = nextSortDir == SWT.UP ? SWT.DOWN : SWT.UP;
			}
			itemsTable.setSortDirection(nextSortDir);
		}

		int index = ITEM_TYPE_COL_NUM;
		int type = STRING;
		if (nextSortColumn == itemCountTableColumn) {
			index = ITEM_COUNT_COL_NUM;
			type = LONG;
		} else if (nextSortColumn == itemSizeTableColumn) {
			index = ITEM_SIZE_COL_NUM;
			type = LONG;
		} else if (nextSortColumn == itemSizeRatioTableColumn) {
			index = ITEM_SIZE_RATIO_COL_NUM;
			type = LONG;
		} else if (nextSortColumn == itemCountTableColumn) {
			index = ITEM_COUNT_COL_NUM;
			type = LONG;
		} else if (nextSortColumn == itemHitsTableColumn) {
			index = ITEM_HITS_COL_NUM;
			type = LONG;
		} else if (nextSortColumn == itemMissesTableColumn) {
			index = ITEM_MISSES_COL_NUM;
			type = LONG;
		} else if (nextSortColumn == itemRefreshesTableColumn) {
			index = ITEM_REFRESHES_COL_NUM;
			type = LONG;
		}
		
		Comparator<TableItem> comparator = getTableItemComparator(type, index, nextSortDir == SWT.UP);

		TableItem[] items = itemsTable.getItems();
		TableItem[] clone = items.clone();
		
		int [] selectedItems = itemsTable.getSelectionIndices();
		IdentityHashMap<TableItem, Object> selection = new IdentityHashMap<TableItem, Object>(selectedItems.length * 4 / 3);
		for (int s : selectedItems) {
			selection.put(items[s], null);
		}
		selectedItems = null;
		
		Arrays.sort(clone, comparator);
		int start;
		for (start = 0; start < clone.length && clone[start] == items[start]; start ++);
		
		for (int j = start; j < clone.length; j++) {
			String[] values = {
					clone[j].getText(SVC_METHOD_COL_NUM),
					clone[j].getText(SVC_CALL_COUNT_COL_NUM),
					clone[j].getText(SVC_ELAPSED_TIME_COL_NUM),
					clone[j].getText(SVC_TIME_RATIO_COL_NUM),
					clone[j].getText(SVC_AVERAGE_TIME_COL_NUM),
					clone[j].getText(SVC_WORST_TIME_COL_NUM),
					clone[j].getText(SVC_TOOLTIP_COL_NUM) };
			TableItem item = new TableItem(itemsTable, SWT.NONE, clone.length + j - start);
			item.setText(values);
			if (selection.containsKey(clone[j])) {
				itemsTable.select(clone.length + j - start);
			}
		}
		for (int j = start; j < clone.length; j++) {
			items[j].dispose();
		}
	}

	private void updateServiceCountTab() {
		if (servicesTree.isDisposed())
			return;
		if (servicesStatsChanged) {
			servicesStatsChanged = false;
			Class<?>[] service = model.getServices(statistics);
			TreeItem[] items = servicesTree.getItems();

			int itemsCount = items.length;
			boolean allExpanded = (itemsCount > 0) ? true : false;
			for (int i = 0; i < itemsCount; i++) {
				allExpanded = items[i].getExpanded();
				if (!allExpanded)
					break;
			}
			
			HashMap<String, TreeItem> itemsMap = new HashMap<String, TreeItem>(items.length * 4 / 3);
			for (int i = 0; i < items.length; i++) {
				itemsMap.put(items[i].getText(SVC_TOOLTIP_COL_NUM), items[i]);
			}
			items = null;
			
			HashMap<String, TreeItem> subItemsMap = new HashMap<String, TreeItem>();


			for (int i = 0; i < service.length; i++) {
				Class<?> s = service[i];
				if (model.getServiceCallCount(statistics, s) > 0) {
					TreeItem item = itemsMap.get(s.getName());
					boolean found = true;
					if (item == null) {
						item = new TreeItem(servicesTree, SWT.NONE);
						found = false;
					}
					item.setText(model.getServiceData(statistics, s));

					ITheme theme = PlatformUI.getWorkbench()
							.getThemeManager().getCurrentTheme();
					FontRegistry fontRegistry = theme.getFontRegistry();
					Font font = fontRegistry
							.getBold(JFaceResources.DEFAULT_FONT);
					item.setFont(font);

					Method[] method = model.getMethods(statistics, s);
					TreeItem[] subItems = item.getItems();
					subItemsMap.clear();
					for (TreeItem subItem : subItems) {
						subItemsMap.put(subItem.getText(0), subItem);
					}
					for (int k = 0; k < method.length; k++) {
						Method m = method[k];
						if ((model.getMethodCallCount(statistics, s, m) > 0)
								|| (model.getMethodElapsedTime(
										statistics, s, m) > 0)) {
							TreeItem subItem = subItemsMap.get(model.getMethodName(m));
							if (subItem == null) {
								subItem = new TreeItem(item, SWT.NONE);
							}
							subItem.setText(model.getMethodData(
									statistics, s, m));
						}
					}
					if (!found && allExpanded) {
						item.setExpanded(true);
					}
				}
			}
			servicesTree.layout();

			if (servicesTree.getSortColumn() != null)
				sortServicesTable(null);

			getStatusLineManager().setMessage(null,
					getStatus(statistics));
		} else {
			// reset
			servicesTree.removeAll();
			getStatusLineManager().setMessage(null,
					getStatus(statistics));
		}

		setEnabledForSelectedTab();
	}

	private void updateItemManagerTab() {
		if (itemsTable.isDisposed()) {
			return;
		}
		
		itemsStatsChanged = false;
		
		IItemType[] itemTypes = model.getItemTypes(statistics);
		TableItem[] items = itemsTable.getItems();
		HashMap<String, TableItem> itemsMap = new HashMap<String, TableItem>(items.length * 4 / 3);
		for (int i = 0; i < items.length; i++) {
			itemsMap.put(items[i].getText(0), items[i]);
		}
		
		for (int i = 0; i < itemTypes.length; i++) {
			IItemType itemType = itemTypes[i];
			TableItem item = itemsMap.get(model.getItemTypeName(itemType));
			if ((model.getItemCount(statistics, itemType) > 0)
					|| (model.getItemHits(statistics, itemType) > 0)
					|| (model.getItemMisses(statistics, itemType) > 0)
					|| (model
							.getItemRefreshes(statistics, itemType) > 0)
					|| (model.getItemSize(statistics, itemType) > 0)) {
				if (item == null) {
					item = new TableItem(itemsTable, SWT.NONE);
				}
				item.setText(model.getItemTypeData(statistics,
						itemType));
			} else if (item != null) {
				item.dispose();
			}
		}
		itemsTable.layout();

		if (itemsTable.getSortColumn() != null)
			sortItemsTable(null);

		getStatusLineManager().setMessage(null,
				getStatus(statistics));

		setEnabledForSelectedTab();
	}

	private void resizeServicesTree(final Composite composite) {
		Rectangle area = composite.getClientArea();

		Point preferredSize = servicesTree
				.computeSize(SWT.DEFAULT, SWT.DEFAULT);
		int width = area.width - 2 * servicesTree.getBorderWidth();
		if (preferredSize.y > area.height + servicesTree.getHeaderHeight()) {
			// Subtract the scrollbar width from the total column width
			// if a vertical scrollbar will be required
			Point vBarSize = servicesTree.getVerticalBar().getSize();
			width -= vBarSize.x;
		}
		Point oldSize = servicesTree.getSize();
		if (oldSize.x > area.width) {
			// table is getting smaller so make the columns
			// smaller first and then resize the table to
			// match the client area width
			serviceMethodTreeColumn.setWidth(5 * width / 10);
			svcCallCountTreeColumn.setWidth(1 * width / 10);
			svcTimeTreeColumn.setWidth(1 * width / 10);
			svcTimeRatioTreeColumn.setWidth(1 * width / 10);
			srcAvgTimeTreeColumn.setWidth(1 * width / 10);
			srcWorstTimeTreeColumn.setWidth(1 * width / 10);
			servicesTree.setSize(area.width, area.height);
		} else {
			// table is getting bigger so make the table
			// bigger first and then make the columns wider
			// to match the client area width
			servicesTree.setSize(area.width, area.height);
			serviceMethodTreeColumn.setWidth(5 * width / 10);
			svcCallCountTreeColumn.setWidth(1 * width / 10);
			svcTimeTreeColumn.setWidth(1 * width / 10);
			svcTimeRatioTreeColumn.setWidth(1 * width / 10);
			srcAvgTimeTreeColumn.setWidth(1 * width / 10);
			srcWorstTimeTreeColumn.setWidth(1 * width / 10);
		}
	}

	private void resizeItemsTable(final Composite composite) {
		Rectangle area = composite.getClientArea();
		Point preferredSize = itemsTable.computeSize(SWT.DEFAULT, SWT.DEFAULT);
		int width = area.width - 2 * itemsTable.getBorderWidth();
		if (preferredSize.y > area.height + itemsTable.getHeaderHeight()) {
			// Subtract the scrollbar width from the total column width
			// if a vertical scrollbar will be required
			Point vBarSize = itemsTable.getVerticalBar().getSize();
			width -= vBarSize.x;
		}
		Point oldSize = itemsTable.getSize();
		if (oldSize.x > area.width) {
			// table is getting smaller so make the columns
			// smaller first and then resize the table to
			// match the client area width
			itemTypeTableColumn.setWidth(3 * width / 10);
			itemCountTableColumn.setWidth(1 * width / 10);
			itemSizeTableColumn.setWidth(1 * width / 10);
			itemSizeRatioTableColumn.setWidth(1 * width / 10);
			itemHitsTableColumn.setWidth(1 * width / 10);
			itemMissesTableColumn.setWidth(1 * width / 10);
			itemRefreshesTableColumn.setWidth(1 * width / 10);
			itemsTable.setSize(area.width, area.height);
		} else {
			// table is getting bigger so make the table
			// bigger first and then make the columns wider
			// to match the client area width
			itemsTable.setSize(area.width, area.height);
			itemTypeTableColumn.setWidth(3 * width / 10);
			itemCountTableColumn.setWidth(1 * width / 10);
			itemSizeTableColumn.setWidth(1 * width / 10);
			itemSizeRatioTableColumn.setWidth(1 * width / 10);
			itemHitsTableColumn.setWidth(1 * width / 10);
			itemMissesTableColumn.setWidth(1 * width / 10);
			itemRefreshesTableColumn.setWidth(1 * width / 10);
		}
	}
	
	
	private void copyToClipboard() {
		StringBuffer sb = new StringBuffer();
		if (cb == null)
			cb = new Clipboard(getShell().getDisplay());
		int index = tabFolder.getSelectionIndex();
		String lineSeparator = System.getProperty("line.separator"); //$NON-NLS-1$
		if (index == 0) {
			copyServices(sb, lineSeparator);
		} else if(index == 1) {
			copyItems(sb, lineSeparator);
		} else if(index == 2) {
			sb.append(NLS.bind(Messages.MetronomeWindow_pingTimePasteMessage + lineSeparator, pingTimeValue.getText()));
			sb.append(NLS.bind(Messages.MetronomeWindow_uploadSpeedPasteMessage + lineSeparator, upBandwidthValue.getText()));
			sb.append(NLS.bind(Messages.MetronomeWindow_downloadSpeedPasteMessage + lineSeparator, downBandwidthValue.getText()));
		}

		if (sb.length() > 0) {
			TextTransfer textTransfer = TextTransfer.getInstance();
			cb.setContents(new Object[] { sb.toString() },
					new Transfer[] { textTransfer });
		}
	}
	
	private void copyItems(StringBuffer sb, String lineSeparator) {
		String formatString = Messages.MetronomeWindow_itemTypeFormat;
		sb.append(NLS.bind(formatString, new String[] {Messages.MetronomeWindow_itemTypeColumnHeading, Messages.MetronomeWindow_countColumnHeading, Messages.MetronomeWindow_sizeBytesColumnHeading, Messages.MetronomeWindow_sizePercentColumnHeading, Messages.MetronomeWindow_hitColumnHeading, Messages.MetronomeWindow_missColumnHeading, Messages.MetronomeWindow_refreshColumnHeading})
				+ lineSeparator);
		
		TableItem[] items = itemsTable.getItems();
		int [] indices = itemsTable.getSelectionIndices();
		Arrays.sort(indices);
		for (int i = 0; i < indices.length; i++) {
			TableItem item = items[indices[i]];
			sb.append(NLS.bind(formatString + lineSeparator, new String[] {
					item.getText(0),
					item.getText(1),
					item.getText(2),
					item.getText(3),
					item.getText(4),
					item.getText(5),
					item.getText(6)
			}));
		} 
	}
	
	private void copyServices(StringBuffer sb, String lineSeparator) {
		String formatString = Messages.MetronomeWindow_serviceMethodCopyTextFormat;
		
		sb.append(NLS.bind(formatString, new String[] {
				Messages.MetronomeWindow_serviceMethodColumnHeading, Messages.MetronomeWindow_countColumnHeading, Messages.MetronomeWindow_timeSecondsColumnHeading, Messages.MetronomeWindow_timePercentColumnHeading, Messages.MetronomeWindow_timeAverageColumnHeading, Messages.MetronomeWindow_timeWorstColumnHeading})
				+ lineSeparator);
		TreeItem[] items = servicesTree.getSelection();
		IdentityHashMap<TreeItem, Object> selection = new IdentityHashMap<TreeItem, Object>(items.length * 4 / 3);
		for (TreeItem item : items) {
			selection.put(item, item);
		}
		for (TreeItem item : servicesTree.getItems()) {
			boolean writtenLines = false;
			if (selection.containsKey(item)) {
				sb.append(NLS.bind(formatString, new String[] {
						item.getText(0),
						item.getText(1),
						item.getText(2),
						item.getText(3),
						item.getText(4),
						item.getText(5)}));
				sb.append(lineSeparator);
				writtenLines = true;
			}
			
			for (TreeItem child : item.getItems()) {
				if (selection.containsKey(child)) {
					sb.append(NLS.bind("  " + formatString, new String[] { //$NON-NLS-1$
							child.getText(0),
							child.getText(1),
							child.getText(2),
							child.getText(3),
							child.getText(4),
							child.getText(5)}));
					sb.append(lineSeparator);
					writtenLines = true;
				}
			}
			
			if (writtenLines) {
				sb.append(lineSeparator);
			}
		}
	}

	private void expandServicesTree() {
		TreeItem[] items = servicesTree.getItems();
		for (int i = 0; i < items.length; i++) {
			items[i].setExpanded(true);
		}
	}

	private void collapseServicesTree() {
		TreeItem[] items = servicesTree.getItems();
		for (int i = 0; i < items.length; i++) {
			items[i].setExpanded(false);
		}
	}

	private void resetServicesStats() {
		model.resetServiceMethodStats(statistics);
		updateServiceCountTab();
		setEnabledForSelectedTab();
	}

	private void resetItemsStats() {
		model.resetItemTypeStats(statistics);
		updateItemManagerTab();
		setEnabledForSelectedTab();
	}

	private void setEnabledForServicesButtons() {
		if (servicesTree.getItemCount() > 0) {
			expandAllButton.setEnabled(true);
			collapseAllButton.setEnabled(true);
			resetServicesButton.setEnabled(true);
		} else {
			expandAllButton.setEnabled(false);
			collapseAllButton.setEnabled(false);
			resetServicesButton.setEnabled(false);
		}
	}

	private void setEnabledForItemsButtons() {
		if (itemsTable.getItemCount() > 0) {
			resetItemsButton.setEnabled(true);
		} else {
			resetItemsButton.setEnabled(false);
		}
	}

	private void setEnabledForServicesActions() {
		if (servicesTree.getItemCount() > 0) {
			expandAllAction.setEnabled(true);
			collapseAllAction.setEnabled(true);
			resetAction.setEnabled(true);
		} else {
			expandAllAction.setEnabled(false);
			collapseAllAction.setEnabled(false);
			resetAction.setEnabled(false);
		}
	}

	private void setEnabledForItemsActions() {
		if (itemsTable.getItemCount() > 0) {
			resetAction.setEnabled(true);
		} else {
			resetAction.setEnabled(false);
		}
		expandAllAction.setEnabled(false);
		collapseAllAction.setEnabled(false);
	}

	private void setEnabledForCopyServices() {
		if (servicesTree.getSelectionCount() > 0) {
			copyServicesButton.setEnabled(true);
			copyAction.setEnabled(true);
		} else {
			copyServicesButton.setEnabled(false);
			copyAction.setEnabled(false);
		}
	}

	private void setEnabledForCopyItems() {
		if (itemsTable.getSelectionCount() > 0) {
			copyItemsButton.setEnabled(true);
			copyAction.setEnabled(true);
		} else {
			copyItemsButton.setEnabled(false);
			copyAction.setEnabled(false);
		}
	}

	private void setEnabledForSelectedTab() {
		if (selectedTabIndex == 0) {
			setEnabledForServicesButtons();
			setEnabledForServicesActions();
			setEnabledForCopyServices();
		} else {
			setEnabledForItemsButtons();
			setEnabledForItemsActions();
			setEnabledForCopyItems();
		}
	}

	// for Ctrl+C to work
	// workaround until an Eclipse bug (CopyHandler conflicts
	// between two active different shells) is fixed
	private final static class ActiveShellExpression extends
			org.eclipse.core.expressions.Expression {

		/**
		 * The seed for the hash code for all schemes.
		 */
		private static final int HASH_INITIAL = ActiveShellExpression.class
				.getName().hashCode();

		/**
		 * The sources value to use with this expression.
		 */
		public static final int SOURCES = ISources.ACTIVE_SHELL
				| ISources.ACTIVE_WORKBENCH_WINDOW;

		/**
		 * The shell that must be active for this expression to evaluate to
		 * <code>true</code>. If this value is <code>null</code>, then any
		 * shell may be active.
		 */
		private final Shell activeShell;

		/**
		 * Constructs a new instance of <code>ActiveShellExpression</code>
		 * 
		 * @param activeShell
		 *            The shell to match with the active shell;
		 *            <code>null</code> if it will match any active shell.
		 */
		public ActiveShellExpression(final Shell activeShell) {
			this.activeShell = activeShell;
		}

		/**
		 * Expression information for this expression. Namely active shell and
		 * active workbench window name.
		 * 
		 * @since 3.2
		 */
		public final void collectExpressionInfo(final ExpressionInfo info) {
			info.addVariableNameAccess(ISources.ACTIVE_SHELL_NAME);
			info.addVariableNameAccess(ISources.ACTIVE_WORKBENCH_WINDOW_NAME);
			info.addVariableNameAccess(ISources.ACTIVE_CURRENT_SELECTION_NAME);
		}

		protected final int computeHashCode() {
			return HASH_INITIAL * HASH_FACTOR + hashCode(activeShell);
		}

		public final boolean equals(final Object object) {
			if (object instanceof ActiveShellExpression) {
				final ActiveShellExpression that = (ActiveShellExpression) object;
				return equals(this.activeShell, that.activeShell);
			}

			return false;
		}

		/**
		 * Evaluates this expression. If the active shell defined by the context
		 * matches the shell from this expression, then this evaluates to
		 * <code>EvaluationResult.TRUE</code>. Similarly, if the active
		 * workbench window shell defined by the context matches the shell from
		 * this expression, then this evaluates to
		 * <code>EvaluationResult.TRUE</code>.
		 * 
		 * @param context
		 *            The context from which the current state is determined;
		 *            must not be <code>null</code>.
		 * @return <code>EvaluationResult.TRUE</code> if the shell is active;
		 *         <code>EvaluationResult.FALSE</code> otherwise.
		 */
		public final EvaluationResult evaluate(final IEvaluationContext context) {
			if (activeShell != null) {
				Object value = context.getVariable(ISources.ACTIVE_SHELL_NAME);
				if (!activeShell.equals(value)) {
					value = context
							.getVariable(ISources.ACTIVE_WORKBENCH_WINDOW_SHELL_NAME);
					if (!activeShell.equals(value)) {
						return EvaluationResult.FALSE;
					}
				}
			}

			return EvaluationResult.TRUE;
		}

		public final String toString() {
			final StringBuffer buffer = new StringBuffer();
			buffer.append("ActiveShellExpression("); //$NON-NLS-1$
			buffer.append(activeShell);
			buffer.append(')');
			return buffer.toString();
		}
	}

	// for Ctrl+C to work
	public int open() {
		int rc = super.open();
		IContextService contextService = (IContextService) PlatformUI
				.getWorkbench().getService(IContextService.class);
		contextService.registerShell(getShell(), IContextService.TYPE_WINDOW);
		IHandlerService handlerService = (IHandlerService) PlatformUI
				.getWorkbench().getService(IHandlerService.class);
		copyHandler = handlerService.activateHandler(
				"org.eclipse.ui.edit.copy", new ActionHandler(copyAction), //$NON-NLS-1$
				new ActiveShellExpression(getShell()));
		return rc;
	}


	private void updateRepositoryTabFromNonUiThread() {
	    Shell shell = getShell();
		if (shell == null || shell.isDisposed()) {
			return;
		}

		shell.getDisplay().asyncExec(new Runnable() {
			@Override
			public void run() {
		        updateRepositoryTab();
			}
		});
	}
	
	private IListener metronomeRepoListener = new IListener() {
		@Override
		@SuppressWarnings("rawtypes")
		public void handleEvents(List events) {
			updateRepositoryTabFromNonUiThread();
		}
	};
	
	@Override
	public void addedRepository(ITeamRepository repository) {
	    repository.addGenericListener(repository, metronomeRepoListener);
	    updateRepositoryTabFromNonUiThread();
	}

	@Override
	public void removedRepository(ITeamRepository repository) {
	    repository.removeGenericListener(repository, metronomeRepoListener);
	    updateRepositoryTabFromNonUiThread();
	}
}

