/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.metronome.internal;

import org.eclipse.ui.IWindowListener;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import com.ibm.team.filesystem.ui.UiPlugin;
import com.ibm.team.metronome.MetronomePlugin;

/**
 * Class responsible for setting the trim group's visibility on startup. 
 * Since the trim group may not be known to the TrimManager, we associate a 
 * listener with the workbench to prevent the trim's display at the last 
 * minute.
 */
public class VisibilityController {
	
	static IWindowListener listener = new IWindowListener() {

		public void windowActivated(IWorkbenchWindow window) {
			boolean result = MetronomePlugin.getDefault().setMetronomeVisible(
					UiPlugin.getBooleanPreference(UiPlugin.SHOW_METRONOME_TRIM));
			
			if (result) {
				PlatformUI.getWorkbench().removeWindowListener(this);
			}
		}

		public void windowClosed(IWorkbenchWindow window) {
		}

		public void windowDeactivated(IWorkbenchWindow window) {
		}

		public void windowOpened(IWorkbenchWindow window) {
		}
		
	};
	
    /**
     * Update the trim's visibility to reflect the current state of the 
     * {@link UiPlugin#SHOW_METRONOME_TRIM} preference.
     */
    public static void updateVisibility() {
        boolean foundGroup = MetronomePlugin.getDefault().setMetronomeVisible(UiPlugin.getBooleanPreference(UiPlugin.SHOW_METRONOME_TRIM));

        // We do the following for the first run of the workbench. On the first 
        // run, the TrimManager doesn't know about the trim group when our 
        // plugin is being initialized. We add a listener that will fire when 
        // the workbench is about to be shown
        if (foundGroup) {
        	PlatformUI.getWorkbench().removeWindowListener(listener);
        }
        else {
        	PlatformUI.getWorkbench().addWindowListener(listener);
        }
    }

	

}
