/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.metronome.traffic;

import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Listener;

import com.ibm.team.metronome.internal.Messages;

/**
 * Custom control for showing server traffic.
 */
public class TrafficControl extends Composite {
	CLabel label;
	TrafficModel traffic;
	int lastHits = -1;
	
	public TrafficControl(Composite parent) {
		super(parent, SWT.NONE);
		GridLayout layout = new GridLayout();
		layout.marginWidth = 0;
		layout.marginHeight = 0;
		layout.marginTop = 5;
		setLayout(layout);
		label = new CLabel(this, SWT.CENTER);
		label.setText(Messages.TrafficControl_serverHits);
		label.setToolTipText(Messages.TrafficControl_serverHitsTooltip);
		label.addListener(SWT.MouseDown, new Listener() {
			public void handleEvent(Event event) {
				lastHits = -1;
				label.setText(Messages.TrafficControl_recentlyClearedMessage);
				traffic.clear();
			}
		});
		label.addListener(SWT.MouseDoubleClick, new Listener() {
			public void handleEvent(Event event) {
				label.setText(Messages.TrafficControl_stats);
			}
		});
		
		GridData layoutData = new GridData(SWT.FILL, SWT.CENTER, true, true);
		GC gc = new GC(label);
		layoutData.minimumWidth = gc.textExtent("88888 Hits").x; // space for up to 100000-1 hits, plus suffix //$NON-NLS-1$
		layoutData.verticalAlignment = GridData.VERTICAL_ALIGN_CENTER;
		gc.dispose();
		label.setLayoutData(layoutData);
		
		traffic = new TrafficModel();
		final Display display = getDisplay();
		display.timerExec(1000, new Runnable() {
			public void run() {
				if (display.isDisposed() || label.isDisposed()) return;
				try {
					int hits = traffic.getHits();
					if (lastHits != hits) {
						lastHits = hits;
						String format = "{0}"; //$NON-NLS-1$
						if (hits == 1)
							format = Messages.TrafficControl_hitSingular;
						else if (hits < 100000) // don't show any suffix if >= 100000
							format = Messages.TrafficControl_hitPlural;
						label.setText(NLS.bind(format, hits));
					}
				} catch (Exception e) {
					label.setText(Messages.TrafficControl_hitError);
					return;
				}
				display.timerExec(1000, this);
			}
		});
	}

}
