/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.metronome.traffic;

import java.util.Enumeration;
import java.util.Hashtable;

import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.client.ITeamRepositoryService;
import com.ibm.team.repository.client.TeamPlatform;
import com.ibm.team.repository.client.ITeamRepositoryService.IRepositoryServiceListener;

public class TrafficModel implements IRepositoryServiceListener {
    Hashtable repositories = new Hashtable();

    public TrafficModel() {
    	ITeamRepositoryService repoService = TeamPlatform.getTeamRepositoryService();
		ITeamRepository[] repositories = repoService.getTeamRepositories();
    	for (int i = 0; i < repositories.length; i++)
    		addedRepository(repositories[i]);
    	repoService.addRepositoryServiceListener(this);
    }

    /* IRepositoryServiceListener */
    
	public void removedRepository(ITeamRepository repository) {
		// remove from the model to avoid leaks (see bug 38210)
		// any further requests to the given repo will not be tracked
		repositories.remove(repository);
	}

	public void addedRepository(ITeamRepository repository) {
		repositories.put(repository, new Stats(repository));
	}

    /* TrafficModel */

    public int getHits() {
        int total = 0;
        Enumeration enumeration = repositories.elements();
        while (enumeration.hasMoreElements()) {
            Stats stats = (Stats) enumeration.nextElement();
            total += stats.repo.statistics().getTotalServiceCallCount()
                    - stats.hitsWhenReset;
        }
        return total;
    }

    public void clear() {
        Enumeration enumeration = repositories.elements();
        while (enumeration.hasMoreElements()) {
            Stats stats = (Stats) enumeration.nextElement();
            stats.hitsWhenReset = stats.repo.statistics().getTotalServiceCallCount();
        }
    }

    public class Stats {
        ITeamRepository repo;

        long hitsWhenReset;

        public Stats(ITeamRepository repo) {
            this.repo = repo;
        }
    }
}
