/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz;

import java.io.IOException;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.scm.client.importz.spi.ChangeSetFileReader;

/**
 * As an import process must be able to handle a vast number of import change sets
 * it is not always possible to hold all necessary information in memory. Thus it is
 * necessary to store the information temporarily on disk. For this purpose we need 
 * <code>IChangeSetFileReader</code> and <code>IChangeSetFileWriter</code>.</p>  
 *
 * <p>
 * An IChangeSetFileReader represents the general interface for reading change set information
 * of a given file.<p>
 * 
 * <p>
 * This interface is not intended to be implemented by clients. Clients can
 * instantiate or subclass the {@link ChangeSetFileReader} class instead.
 * 
 * <p>Note: A concrete log file reader must be created via a corresponding factory. Thus it's
 * necessary that one must implement the contrary writer as well.</p>
 * 
 * @since 0.6
 * 
 * @see com.ibm.team.scm.client.importz.IChangeSetFileWriter
 * @see ChangeSetFileReader
 *      com.ibm.team.migration.ccvs.IMigrationFactory
 */
public interface IChangeSetFileReader {
	
	/**
	 * Close the reader.
	 * @throws IOException 
	 */
	void close();
	
	/**
	 * <p>Reads the next element. Thereby the current change set and the current changes 
	 * are set.</p>
	 * 
	 * <p>Implementation note: If there is no next change set the current change set and 
	 * current changes have to be set null and the access to the file have to be close 
	 * in order to be able to delete the temporary file.</p>  
	 * @param monitor a progress monitor or <code>null</code>
	 * 
	 * @throws IOException if problems occur while accessing the file
	 */
	IImportChangeSet next(IProgressMonitor monitor) throws IOException;
    
	/**
	 * Return whether directories are significant for the change sets being read.
	 * If <code>false</code> is returned, there should be no folder changes included
	 * in the returned change sets. Instead, folder additions and deletions will
	 * be deduced from file level changes.
	 * @return
	 */
    boolean getDirectoriesSignificant();
}
