/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz;

import com.ibm.team.filesystem.common.FileLineDelimiter;
import com.ibm.team.filesystem.common.IFileContent;
import com.ibm.team.scm.client.importz.spi.FileProperties;

/**
 * This interface defines the set of file specific properties that
 * can be associated with a file contained in an {@link IImportChange}.
 * <p>
 * This interface is not intended to be implemented by clients.
 * Clients that need to create an instance of {@link IFileProperties}
 * should use the {@link FileProperties} class.
 * 
 * @since 0.6
 */
public interface IFileProperties {

	/**
	 * Return the line delimiter for the contents of this change.
	 * @return the line delimiter for the contents of this change
	 */
	FileLineDelimiter getLineDelimiter();

	/**
	 * Return the content type for the contents of this change.
	 * @return the content type for the contents of this change
	 * @see IFileContent.CONTENT_TYPE_TEXT
	 * @see IFileContent.CONTENT_TYPE_UNKNOWN
	 */
	String getContentType();

	/**
	 * Return the encoding for the contents of this change.
	 * @return the encoding for the contents of this change
	 * @see IFileContent.ENCODING_US_ASCII
	 */
	String getEncoding();
	
	/**
	 * Return whether the file is expected to be executable or not.
	 * @return whether the file is expected to be executable or not
	 */
	boolean isExecutable();

	/**
	 * Return whether this file's content type is text or not.
	 * @return whether this file's content type is text or not
	 */
	boolean isText();
}
