/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz;

import java.util.Date;
import java.util.List;

import com.ibm.team.scm.client.importz.spi.ImportChangeSet;

/**
 * <p>An <code>IImportChangeSet</code> is the general representation of a change set. An import change set 
 * (independent of the CM system) is specified by its comment, date and author.</p> 
 * 
 * <p>Implementation note: As IImportChangeSets are used inside <code>HashMaps</code> concrete 
 * subclasses must override the methods <code>equals(Object):boolean</code> and 
 * <code>hashCode():int</code>.</p>
 * <p>
 * This interface is not intended to be implemented by clients. Import service providers 
 * should instead use or subclass {@link ImportChangeSet}.
 * 
 * @since 0.6
 */
public interface IImportChangeSet {
	
	/**
	 * Returns the comment of this import change set. 
	 * 
	 * @return comment of the import change set, never null
	 */
	String getComment();
	
	/**
	 * Returns the date of this import change set.
	 * 
	 * @return date of the import change set, never null
	 */
	Date getDate();

	/**
	 * Returns the author of this import change set.
	 * 
	 * @return author of the import change set, never null
	 */
	String getAuthor();
	
	/**
	 * Returns the revision identifier common to all changes contained in the set or
	 * <code>null</code> if changes within this set can have different revisions.
	 * @return the revision identifier common to all changes contained in the set or
	 * <code>null</code>
	 */
	String getRevision();
	
	/**
	 * Return the list of changes contained in this change set.
	 * @return the list of changes contained in this change set
	 */
	List<IImportChange> getChanges();

	/**
	 * Return the comment for this change set with either the
	 * author or date or both embedded in the comment.
	 * @param embedAuthor embed the author
	 * @param embedDate embed the date
	 * @return the comment with the author and/or date embedded
	 */
	String asCommentMessage(boolean embedAuthor, boolean embedDate);
}
