/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz;

import java.io.IOException;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IContributorHandle;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.scm.client.importz.spi.ImportData;

/**
 * Interface that defines the basic import input data.
 * <p>
 * This class is not intended to be implemented by clients. Clients
 * can use or subclass {@link ImportData}.
 * 
 * @since 0.6
 */
public interface IImportData {
	
	/**
	 * Return the contributor handle associated with the given author or <code>null</code>
	 * if there is no contributor for that author id.
	 * @param teamRepository the team repository
	 * @param authorId the author id
	 * @param monitor a progress monitor
	 * @return the contributor handle associated with the given author or <code>null</code>
	 * @throws TeamRepositoryException
	 */
	IContributorHandle getContributor(ITeamRepository teamRepository, String authorId, IProgressMonitor monitor) throws TeamRepositoryException;

	/**
	 * Open the archive connected to the import data. A boolean is returned indicating whether the 
	 * archive was already open. When <code>true</code> is returned, the caller is responsible 
	 * for closing the archive.
	 * @param monitor a progress monitor
	 * @throws IOException 
	 */
	boolean openArchive(IProgressMonitor monitor) throws TeamRepositoryException;

	/**
	 * Close the archive that was previously opened by calling {@link #openArchive(IProgressMonitor)}
	 */
	void closeArchive();

	/**
	 * Return the number of entries in the archive.
	 * @return the number of entries in the archive
	 */
	int getArchiveSize();
	
	/**
	 * Return the name that should be used as the name of the post import baseline.
	 * @return the name of the post import baseline
	 */
	String getPostImportBaselineName();

	/**
	 * Return the encoding that is to be used for any text files for which a sepcific encoding is
	 * not provided.
	 * @return the encoding that is to be used for any text files for which a sepcific encoding is
	 * not provided
	 */
	String getWorkspaceEncoding();

	/**
	 * Indicate whether we are importing over an existing component and want any differences between the
	 * current state of the component and the changes being imported to be reconciled.
	 * @return whether we are importing over an existing component
	 */
	boolean isImportOverExisting();

}
