/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz;

import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.scm.client.IWorkspaceConnection;
import com.ibm.team.scm.client.importz.spi.ImportParticipant;
import com.ibm.team.scm.common.IChangeSetHandle;
import com.ibm.team.scm.common.IComponent;
import com.ibm.team.scm.common.IComponentHandle;
import com.ibm.team.scm.common.IVersionable;
import com.ibm.team.scm.common.IVersionableHandle;
import com.ibm.team.scm.common.dto.IUpdateReport;

/**
 * Allows clients to participate in the import of change set archives.
 * <p>
 * This interface is not intended to be implemented by clients. Clients
 * should instead use or subclass {@link ImportParticipant}.
 * 
 * @since 0.6
 */
public interface IImportParticipant {
	
	/**
	 * Called after an import change set has been read but before the process of committing begins.
	 * @param connection the workspace connection
	 * @param component the component
	 * @param importChangeSet the change set that was read
	 * @param monitor a progress monitor
	 */
	public void changeSetRead(IWorkspaceConnection connection, IComponentHandle component, IImportChangeSet importChangeSet, IProgressMonitor monitor);

	/**
	 * Called after a change set has been committed and closed. The implementer can perform any long running
	 * operation they want at this point.
	 * 
	 * @param connection
	 * the connection used to commit the last change
	 * @param component the component that received the change
	 * @param changeSet
	 * the change set that was committed and closed
	 * @param updateReport
	 * the update report resulting from the commit
	 * @param affectedItems
	 * items that were affected by the commit
	 * @param changes
	 * the list of changes for done in this commit
	 * @param changeLeafs the leaves that were changed
	 * @param monitor a progress monitor
	 */
	public void postCommit(IWorkspaceConnection connection, IComponentHandle component,
			IImportChangeSet importChangeSet, IChangeSetHandle changeSet, IUpdateReport updateReport, IVersionable[] affectedItems, IImportChange[] changes, IVersionableHandle[] changeLeafs, IProgressMonitor monitor);

	/**
	 * Called when the import has finished
	 * @param connection the workspace connection
	 * @param component the component
	 * @param monitor a progress monitor
	 */
	public void postImport(IWorkspaceConnection workspace,
			IComponent component, IProgressMonitor monitor);
}
