/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal;

import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import com.ibm.team.scm.client.importz.IImportChangeSet;

public abstract class AbstractChangeArchiveStatistics implements IChangeArchiveStatistics {

	private Set<String> contributors = new HashSet<String>();
	private int largestChangeSet = 0;
	protected Date firstDate = null;
	protected Date lastDate = null;

	public AbstractChangeArchiveStatistics() {
		super();
	}

	private void addContributor(String contributor) {
		contributors.add(contributor);
	}

	public int getContributorsCount() {
		return contributors.size();
	}

	protected void recordChangeSetSize(int size) {
		if (size > largestChangeSet)
			largestChangeSet = size;
	}

	public int getLargestChangeSetSize() {
		return largestChangeSet;
	}

	protected void recordDate(Date date) {
		if (firstDate == null)
			firstDate = date;
		lastDate = date;
	}

	protected long timeSpan() {
		if (lastDate == null || firstDate == null)
			return 0;
		return lastDate.getTime() - firstDate.getTime();
	}
	
	public void startCommit(IImportChangeSet importChangeSet, long time, boolean preflight) {
	    addContributor(importChangeSet.getAuthor());
	    recordDate(importChangeSet.getDate());
	    recordChangeSetSize(importChangeSet.getChanges().size());
	}

}