/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2009. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IProgressMonitor;

import com.ibm.team.filesystem.client.FileSystemException;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.scm.client.importz.IImportData;
import com.ibm.team.scm.client.importz.IMigrationFactory;

public abstract class AbstractImportOperation {

	private final IMigrationFactory factory;
	private final IImportData importData;
	
	/**
	 * 
	 * @param factory the SCM specific behavior provider for the import
     * @param importData the source SCM specific data that is passed to any
     * 		facilities obtained from the migration factory
	 */
	public AbstractImportOperation(IMigrationFactory factory,
			IImportData importData) {
		Assert.isNotNull(factory);
		Assert.isNotNull(importData);
		this.factory = factory;
		this.importData = importData;
	}

	public IMigrationFactory getFactory() {
		return factory;
	}

	public IImportData getImportData() {
		return importData;
	}
	
	public abstract void run(IProgressMonitor monitor) throws TeamRepositoryException, FileSystemException;
}
