/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal;

import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.ibm.team.scm.client.importz.IImportChange;
import com.ibm.team.scm.client.importz.IImportChangeSet;
import com.ibm.team.scm.client.importz.spi.DerivedImportChangeSet;

public class DerivedChangeSetComparator implements Comparator<IImportChangeSet> {

	public int compare(IImportChangeSet o1, IImportChangeSet o2) {
		return compareChangeSets((DerivedImportChangeSet)o1, (DerivedImportChangeSet)o2);
	}
	
    protected int compareChangeSets(DerivedImportChangeSet cs1, DerivedImportChangeSet cs2) {
        int cmp = cs1.getEarliestDate().compareTo(cs2.getEarliestDate());
        if (cmp != 0) {
            return cmp;
        }
        int finalResult = 0;
        Map<String, String> changeMap = new HashMap<String, String>();
        for (Iterator<IImportChange> i = cs1.getChanges().iterator(); i.hasNext();) {
            IImportChange change = i.next();
            changeMap.put(getPath(change), change.getRevision());
        }
        
        for (Iterator<IImportChange> i = cs2.getChanges().iterator(); i.hasNext();) {
            IImportChange change2 = i.next();
            
            String revision1 = (String)changeMap.get(getPath(change2));
            if (revision1 == null) {
                continue;
            }
            
            String revision2 = change2.getRevision();
            
            int result = compareChangeRevisions(revision1, revision2);
            // Accumulate the result in order to determine whether the result is positive or negative
            finalResult += result;
        }
        
        return finalResult;
	}

	protected String getPath(IImportChange change) {
		if (change.getAfterPath() != null)
			return change.getAfterPath();
		return change.getBeforePath();
	}
	
	protected int compareChangeRevisions(String revision1, String revision2) {
		return revision1.compareTo(revision2);
	}

}
