/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal;

import org.eclipse.osgi.util.NLS;

import com.ibm.team.scm.client.importz.IImportChangeSet;

public class PreFlightChangeArchiveStatistics extends AbstractChangeArchiveStatistics {

	int numChangeSets = 0;
	int numChanges = 0;
	
	public String toDescriptionString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append(NLS.bind(SCMImportMessages.PreFlightChangeArchiveStatistics_0, numChangeSets));
		buffer.append('\n');
		buffer.append(NLS.bind(SCMImportMessages.PreFlightChangeArchiveStatistics_1, numChanges));
		buffer.append('\n');
		buffer.append(NLS.bind(SCMImportMessages.PreFlightChangeArchiveStatistics_2, getContributorsCount()));
		buffer.append('\n');
		buffer.append(NLS.bind(SCMImportMessages.PreFlightChangeArchiveStatistics_3, getLargestChangeSetSize()));
		buffer.append('\n');
		long timeSpan = timeSpan();
		long diffHours = timeSpan / (60 * 60 * 1000);
		long diffDays = timeSpan / (24 * 60 * 60 * 1000);
		long diffYears = diffDays / 365;
		long diffYearsRemain = Math.round(((double)(diffDays % 365) / 365.0) * 100);
		buffer.append(NLS.bind(SCMImportMessages.PreFlightChangeArchiveStatistics_4, new Object[] { diffYears, diffYearsRemain, diffDays, diffHours}));
		buffer.append('\n');
		return buffer.toString();
	}

	public int getChangeSetCount() {
		return numChangeSets;
	}

	public int getChangesCount() {
		return numChanges;
	}

	public void startCommit(IImportChangeSet importChangeSet, long time, boolean preflight) {
		super.startCommit(importChangeSet, time, preflight);
		numChangeSets++;
		numChanges += importChangeSet.getChanges().size();
	}
	
	public void changesCollected(long time) {
		// We're pre-flight so this will never get called
	}

	public void changesCommited(long time) {
		// We're pre-flight so this will never get called
	}

	public void close() {
		// We're pre-flight so this will never get called
	}

	public void endCommit(IImportChangeSet importChangeSet) {
		// We're pre-flight so this will never get called
	}

	public void endPhase(String statisticsName) {
		// We're pre-flight so this will never get called
	}

	public void startPhase(String statisticsName) {
		// We're pre-flight so this will never get called
	}
}
