/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.ui;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;

public abstract class AbstractImportWizardPage extends AbstractWizardPage implements IPropertyChangeListener {

	private List<AbstractOptionsArea> areas = new ArrayList<AbstractOptionsArea>();
	
	public List<AbstractOptionsArea> getAreas() {
		return areas;
	}

	protected AbstractImportWizardPage(String pageName, String title, ImageDescriptor titleImage) {
		super(pageName, title, titleImage);
	}

	@Override
	protected IStatus validatePageState() {
		return getConfiguration().validateState();
	}
	
	protected abstract ImportConfiguration getConfiguration();
	
	protected void addArea(AbstractOptionsArea area) {
		areas.add(area);
	}
	
	@Override
	protected void createPageBody(Composite composite) {
		composite.setLayout(new GridLayout(1, false));
        createPageAreas(composite);
        initControls();
	}
	
	protected abstract void createPageAreas(Composite composite);
	
	protected void initControls() {
		for (AbstractOptionsArea area : areas) {
			area.initControls();
		}
        getConfiguration().addPropertyChangeListener(this);
        updateEnablements();
	}
	
	public void propertyChange(PropertyChangeEvent event) {
		updateEnablements();
	}
	
    @Override
    public void dispose() {
    	super.dispose();
    	getConfiguration().removePropertyChangeListener(this);
    }
	
}
