/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.ui;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.wizard.IWizardPage;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.PlatformUI;

import com.ibm.team.repository.rcp.ui.wizards.BaseWizardPage;

public abstract class AbstractWizardPage extends BaseWizardPage {

	private boolean okToShowError;

	protected AbstractWizardPage(String pageName, String title, ImageDescriptor titleImage) {
		super(pageName, title, titleImage);
	}
	
	public void updateEnablements() {
		IStatus status = validatePageState();
		if (status.isOK()) {
			setMessage(null);
		} else {
			if (okToShowError)
				setMessage(status.getMessage(), getMessageType(status));
		}
        setPageComplete(status.getSeverity() != IStatus.ERROR);
    }

	protected abstract IStatus validatePageState();

	@Override
	public void setVisible(boolean visible) {
		super.setVisible(visible);
		okToShowError = true;
	}
	
	protected int getMessageType(IStatus status) {
		if (status.getSeverity() == IStatus.ERROR)
			return IMessageProvider.ERROR;
		if (status.getSeverity() == IStatus.WARNING)
			return IMessageProvider.WARNING;
		return IMessageProvider.INFORMATION;
	}
	
	@Override
	public IWizardPage getNextPage() {
		if (getWizard() instanceof IAdvanceableWizard) {
			return ((IAdvanceableWizard)getWizard()).getNextPage(this, true);
		}
		return super.getNextPage();
	}
	
	@Override
	public boolean canFlipToNextPage() {
		if (getWizard() instanceof IAdvanceableWizard) {
			return isPageComplete() && ((IAdvanceableWizard)getWizard()).getNextPage(this, false) != null;
		}
		return super.canFlipToNextPage();
	}
	
	@Override
	protected final void createBody(Composite parent) {
		createPageBody(parent);
		String helpContextId = getHelpContextId();
		if (helpContextId != null)
			PlatformUI.getWorkbench().getHelpSystem().setHelp(parent, helpContextId);
		Dialog.applyDialogFont(parent);
	}

	protected String getHelpContextId() {
		return null;
	}

	protected abstract void createPageBody(Composite parent);

}
