/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.ui;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.ibm.team.scm.client.importz.internal.SCMImportMessages;

public class ArchiveFileOptions extends AbstractOptionsArea {

	private Text filenameText;
	private Group fileGroup;
	private final ImportConfiguration configuration;
	private Button useTempArchiveOption;
	private Button useCustomFileOption;
	private Button browseButton;
	private Label filenameLabel;
    
	public ArchiveFileOptions(Composite parent, final ImportConfiguration configuration) {
		this.configuration = configuration;
		
		fileGroup = createGroup(parent, SCMImportMessages.ArchiveFileOptions_0, 3);
		
		if (!configuration.isImportFromArchive()) {
	        useTempArchiveOption = new Button(fileGroup, SWT.RADIO);
	        useTempArchiveOption.setText(SCMImportMessages.ArchiveFileOptions_1);
            GridData data = new GridData(GridData.FILL, GridData.FILL, true, false);
            data.horizontalSpan = 3;
	        useTempArchiveOption.setLayoutData(data);
	        useTempArchiveOption.addSelectionListener(new SelectionAdapter() {
	            public void widgetSelected(SelectionEvent e) {
	            	configuration.setDeleteArchiveFile(useTempArchiveOption.getSelection());
	            }
	        });
	        
	        useCustomFileOption = new Button(fileGroup, SWT.RADIO);
	        useCustomFileOption.setText(SCMImportMessages.ArchiveFileOptions_2);
            data = new GridData(GridData.FILL, GridData.FILL, true, false);
            data.horizontalSpan = 3;
	        useCustomFileOption.setLayoutData(data);
	        useCustomFileOption.addSelectionListener(new SelectionAdapter() {
	            public void widgetSelected(SelectionEvent e) {
	            	configuration.setDeleteArchiveFile(useTempArchiveOption.getSelection());
	            }
	        });
		}
        
		filenameLabel = new Label(fileGroup, SWT.NONE);
		filenameLabel.setText(SCMImportMessages.ArchiveFileOptions_3);
		filenameLabel.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, false, false));
        
        // file name text field
        filenameText = new Text(fileGroup, SWT.SINGLE | SWT.BORDER);
        filenameText.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, false));
        filenameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				configuration.setArchiveFilename(filenameText.getText());
			}
		});
        
        browseButton = new Button(fileGroup, SWT.PUSH);
		browseButton.setText(SCMImportMessages.ArchiveFileOptions_4);
        browseButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				promptForFile();
			}
        });
        browseButton.setLayoutData(new GridData(GridData.FILL, GridData.FILL, false, false));
        final IPropertyChangeListener propertyChangeListener = new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				if (event.getProperty().equals(ImportConfiguration.DELETE_ARCHIVE_FILE)) {
					if (!configuration.isDeleteArchiveFile() && useTempArchiveOption.getSelection()) {
						useTempArchiveOption.setSelection(false);
						useCustomFileOption.setSelection(true);
					}
				}
				updateEnablements();
			}
		};
		configuration.addPropertyChangeListener(propertyChangeListener);
        parent.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				configuration.removePropertyChangeListener(propertyChangeListener);
			}
		});
        updateEnablements();
	}

    public void initControls() {
    	filenameText.setText(configuration.getArchiveFilename());
    	if (useTempArchiveOption != null) {
    		useTempArchiveOption.setSelection(configuration.isDeleteArchiveFile());
    		useCustomFileOption.setSelection(!configuration.isDeleteArchiveFile());
    	}
  }
    
	protected void updateEnablements() {
		enableGroup(fileGroup, isEnabled() && configuration.isImportFullHistory());
		if (isEnabled() && configuration.isImportFullHistory() && useTempArchiveOption != null) {
			if (configuration.isDeleteArchiveFile()) {
				filenameLabel.setEnabled(false);
				filenameText.setEnabled(false);
				browseButton.setEnabled(false);
			} else if (!configuration.isImportIntoWorkspace()) {
				useTempArchiveOption.setEnabled(false);
			}
		}
	}

	private void promptForFile() {
		FileDialog dialog = new FileDialog(fileGroup.getShell(), configuration.isImportFromArchive() ? SWT.OPEN : SWT.SAVE);
		dialog.setFileName(filenameText.getText());
		dialog.setFilterExtensions(new String[] {"*.tar"}); //$NON-NLS-1$
		dialog.setFilterNames(new String[] {SCMImportMessages.ArchiveFileOptions_6});
		String path = dialog.open();
		if (path != null)
			filenameText.setText(path);
	}
}
