/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2010. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.ui;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.DialogPage;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.rcp.common.collection.SetDiff;
import com.ibm.team.repository.rcp.ui.internal.viewers.ISetListener;
import com.ibm.team.scm.client.importz.internal.SCMImportMessages;

public class Author2ContributorMappingPage extends AbstractImportWizardPage implements IPropertyChangeListener {

	public static final String AUTHOR_MAPPING_PROPERTY = "PromptForAuthrMappings"; //$NON-NLS-1$
	public static final String PROMPT_FOR_MAPPINGS = "prompt"; //$NON-NLS-1$
	public static final String NO_MAPPING = "none"; //$NON-NLS-1$
	public static final String USE_PROVIDED_MAPPINGS = "provided"; //$NON-NLS-1$
	public static final String MAP_MATCHING_IDS = "auto"; //$NON-NLS-1$
	
	private final ImportConfiguration configuration;
	private Button promptForMapping;
	private Button useEnteredMappings;
	private Button noMapping;
	private Author2ContributorMappingArea mappingArea;
	private final boolean duringImport;
	private boolean isAdmin;
	private Button autoMapping;

	public Author2ContributorMappingPage(String pageName,
			ImageDescriptor titleImage, ImportConfiguration configuration, boolean duringImport, boolean isAdmin) {
		super(pageName, SCMImportMessages.Author2ContributorMappingPage_4, titleImage);
		this.configuration = configuration;
		this.duringImport = duringImport;
		this.isAdmin = isAdmin;
		setDescription(NLS.bind(SCMImportMessages.Author2ContributorMappingPage_5, configuration.getSourceRepositoryKind()));
		configuration.addPropertyChangeListener(this);
	}

	@Override
	public void dispose() {
		super.dispose();
		configuration.removePropertyChangeListener(this);
	}
	
	@Override
	protected ImportConfiguration getConfiguration() {
		return configuration;
	}

	@Override
	protected void createPageAreas(Composite composite) {
		if (!duringImport) {
	        promptForMapping  = new Button(composite, SWT.RADIO | SWT.WRAP);
	        promptForMapping.setText(NLS.bind(SCMImportMessages.Author2ContributorMappingPage_6, configuration.getSourceRepositoryKind()));
	        promptForMapping.setToolTipText(NLS.bind(SCMImportMessages.Author2ContributorMappingPage_7, configuration.getSourceRepositoryKind()));
	        GridData data = new GridData(GridData.FILL, GridData.FILL, true, false);
	        data.horizontalSpan = 2;
	        promptForMapping.setLayoutData(data);
	        promptForMapping.addSelectionListener(new SelectionAdapter() {
	            public void widgetSelected(SelectionEvent e) {
	            	updateMappingType();
	            }
	        });
		}
        
        noMapping = new Button(composite, SWT.RADIO | SWT.WRAP);
        noMapping.setText(NLS.bind(SCMImportMessages.Author2ContributorMappingPage_8, configuration.getSourceRepositoryKind()));
        GridData data = new GridData(GridData.FILL, GridData.FILL, true, false);
        data.horizontalSpan = 2;
        noMapping.setLayoutData(data);
        noMapping.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
            	updateMappingType();
            }
        });
        
        autoMapping  = new Button(composite, SWT.RADIO | SWT.WRAP);
        autoMapping.setText(NLS.bind(SCMImportMessages.Author2ContributorMappingPage_0, configuration.getSourceRepositoryKind()));
        autoMapping.setToolTipText(NLS.bind(SCMImportMessages.Author2ContributorMappingPage_1, configuration.getSourceRepositoryKind()));
        data = new GridData(GridData.FILL, GridData.FILL, true, false);
        data.horizontalSpan = 2;
        autoMapping.setLayoutData(data);
        autoMapping.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
            	updateMappingType();
            }
        });
        
        useEnteredMappings = new Button(composite, SWT.RADIO | SWT.WRAP);
        useEnteredMappings.setText(SCMImportMessages.Author2ContributorMappingPage_9);
        data = new GridData(GridData.FILL, GridData.FILL, true, false);
        data.horizontalSpan = 2;
        useEnteredMappings.setLayoutData(data);
        useEnteredMappings.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
            	updateMappingType();
            }
        });
        
        mappingArea = new Author2ContributorMappingArea(composite, configuration, !duringImport, getContainer(), getUserOperationRunner());
        mappingArea.getControl().setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
        mappingArea.getAuthorMappings().addListener(new ISetListener<Author2ContributorMapping>() {
			public void changed(SetDiff<Author2ContributorMapping> diff) {
			    updateEnablements();
			}
		});
	}

	protected void initControls() {
		String property = (String)configuration.getProperty(AUTHOR_MAPPING_PROPERTY);
		if (isAdmin) {
			if (property == null) {
				property = duringImport ? USE_PROVIDED_MAPPINGS : PROMPT_FOR_MAPPINGS;
			} else if (property.equals(PROMPT_FOR_MAPPINGS) && duringImport) {
				property = USE_PROVIDED_MAPPINGS;
			}
			setMessage(null);
		} else {
			property = NO_MAPPING;
		}
		if (promptForMapping != null) {
			promptForMapping.setSelection(property.equals(PROMPT_FOR_MAPPINGS));
			promptForMapping.setEnabled(isAdmin);
		}
		if (autoMapping != null) {
			autoMapping.setSelection(property.equals(MAP_MATCHING_IDS));
			autoMapping.setEnabled(isAdmin);
		}
		noMapping.setSelection(property.equals(NO_MAPPING));
		useEnteredMappings.setSelection(property.equals(USE_PROVIDED_MAPPINGS));
		useEnteredMappings.setEnabled(isAdmin);
		super.initControls();
		updateEnablements();
	}

	void updateMappingType() {
		String value = USE_PROVIDED_MAPPINGS;
		if (promptForMapping != null && promptForMapping.getSelection()) {
			value = PROMPT_FOR_MAPPINGS;
		} else if (autoMapping != null && autoMapping.getSelection()) {
			value = MAP_MATCHING_IDS;
		} else if (noMapping.getSelection()) {
			value = NO_MAPPING;
		}
		configuration.setProperty(AUTHOR_MAPPING_PROPERTY, value);
		updateEnablements();
	}
	
	@Override
	public void updateEnablements() {
		if (isComplete()) {
			super.updateEnablements();
		} else {
			setPageComplete(false);
		}
		mappingArea.setEnabled(useEnteredMappings.getSelection());
		if (isVisible() && useEnteredMappings.getSelection()) {
			mappingArea.linkInputToViewer(configuration.getRepository(), getUserOperationRunner());
		}
		if (!isAdmin && getMessage() == null) {
			setMessage(SCMImportMessages.Author2ContributorMappingPage_3, DialogPage.WARNING);
		}
	}
	
	@Override
	protected IStatus validatePageState() {
		return Status.OK_STATUS;
	}
	
	private boolean isVisible() {
		return getControl() != null && getControl().isVisible();
	}

	public boolean isComplete() {
		return mappingArea != null 
			&& (!getAuthorMappings().isEmpty()
				|| (promptForMapping != null && promptForMapping.getSelection())
				|| (autoMapping != null && autoMapping.getSelection())
				|| noMapping.getSelection());
	}

	public Collection<Author2ContributorMapping> getAuthorMappings() {
		if (useEnteredMappings.getSelection())
			return mappingArea.getAuthorMappings().toCollection();
		return new ArrayList<Author2ContributorMapping>();
	}

	public void updateMappingsFor(Collection<String> authorsForSelectedFolders) {
		mappingArea.ensureAuthorIdsAreAvailable(authorsForSelectedFolders);
	}
	
	@Override
	public void setVisible(boolean visible) {
		super.setVisible(visible);
		if (visible && useEnteredMappings.getSelection()) {
			mappingArea.linkInputToViewer(configuration.getRepository(), getUserOperationRunner());
		}
	}

	public void propertyChange(PropertyChangeEvent event) {
		if (event.getProperty().equals(ImportConfiguration.TEAM_REPOSITORY_PROPERTY)) {
			try {
				isAdmin = false;
				getContainer().run(true, false, new IRunnableWithProgress() {
					public void run(IProgressMonitor monitor) throws InvocationTargetException,
							InterruptedException {
						ITeamRepository repository = configuration.getRepository();
						if (repository != null) {
							isAdmin = Author2ContributorMappingArea.hasJazzAdminPermissions(repository, monitor);
						}
					}
				
				});
			} catch (InvocationTargetException e) {
				// Ignore and assume no admin privileges
			} catch (InterruptedException e) {
				// Ignore and assume no admin privileges
			}
			initControls();
		}
	}
	
	@Override
	protected String getHelpContextId() {
		return IHelpContextIds.HELP_CONTEXT_USER_MAPPING_PAGE;
	}

}
