/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.ui;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.ibm.team.scm.client.importz.internal.SCMImportMessages;

/**
 * Options related to creating baselines
 */
public class BaselineOptions extends AbstractOptionsArea {

	private ImportConfiguration configuration;
	private Group baselineGroup;
	private Label nameLabel;
	private Text nameText;
	private Button createButton;

	public BaselineOptions(Composite parent,
			ImportConfiguration configuration) {
		this.configuration = configuration;
		
		baselineGroup = createGroup(parent, SCMImportMessages.BaselineOptions_0, 2);
		
		nameLabel = new Label(baselineGroup, SWT.NONE);
		nameLabel.setText(SCMImportMessages.BaselineOptions_1);
		nameLabel.setLayoutData(new GridData(GridData.FILL, GridData.CENTER, false, false));
        
        // file name text field
        nameText = new Text(baselineGroup, SWT.SINGLE | SWT.BORDER);
        nameText.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, false));
        nameText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				String name = nameText.getText();
				if (name.length() == 0)
					name = null;
				BaselineOptions.this.configuration.setPostImportBaselineName(name);
			}
		});
        
        if (configuration.isIncrementalBaselinesSupported()) {
            createButton = new Button(baselineGroup, SWT.CHECK);
            createButton.setText(SCMImportMessages.BaselineOptions_2);
            createButton.addSelectionListener(new SelectionAdapter() {
    			public void widgetSelected(SelectionEvent e) {
    				BaselineOptions.this.configuration.setCreateIncrementalBaselines(createButton.getSelection());
    			}
            });
        	GridDataFactory.defaultsFor(createButton)
	    		.grab(true, false)
	    		.span(2, 1)
	    		.applyTo(createButton);
        }
	}
	
	@Override
	public void initControls() {
		super.initControls();
		String name = configuration.getPostImportBaselineName();
		if (name == null)
			name = ""; //$NON-NLS-1$
		nameText.setText(name);
		if (createButton != null) {
			createButton.setSelection(configuration.isCreateIncrementalBaselines());
		}
	}

	protected void updateEnablements() {
		boolean enabled = isEnabled();
		nameLabel.setEnabled(enabled);
		nameText.setEnabled(enabled);
		if (createButton != null)
			createButton.setEnabled(enabled);
	}

}
