/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.ui;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.swt.widgets.Composite;

/**
 * Page for configuring the input to the import process
 */
public class ConfigureImportInputPage extends AbstractImportWizardPage implements IPropertyChangeListener {

	private final ImportConfiguration configuration;

	protected ConfigureImportInputPage(String pageName, String title,
			ImageDescriptor titleImage, ImportConfiguration configuration) {
		super(pageName, title, titleImage);
		this.configuration = configuration;
	}

	protected void createPageAreas(Composite composite) {
		if (isIncludeFolderMappingOptions()) {
			FolderMappingOptions folderMappingOptions = new FolderMappingOptions(composite, configuration);
			addArea(folderMappingOptions);
		}
		ImportTargetOptions targetOptions = new ImportTargetOptions(composite, configuration);
		addArea(targetOptions);
		ArchiveFileOptions fileOptions = new ArchiveFileOptions(composite, configuration);
		addArea(fileOptions);
	}
	
	/**
	 * Indicate whether the folder mapping options for a single folder selection 
	 * should be shown (e.g. map folder to top-level folder or component root).
	 * The default is <code>true</code>.
	 * @return whether the folder mapping options for a single folder selection 
	 * should be shown
	 */
	protected boolean isIncludeFolderMappingOptions() {
		return true;
	}

	public boolean canFlipToNextPage() {
		return super.canFlipToNextPage() && configuration.isImportIntoWorkspace();
	}

	@Override
	protected ImportConfiguration getConfiguration() {
		return configuration;
	}
	
    protected boolean isArchiveFile() {
    	return false;
    }
	
	@Override
	protected String getHelpContextId() {
		return IHelpContextIds.HELP_CONTEXT_CONFIGURE_IMPORT_PAGE;
	}
	
	@Override
	public void updateEnablements() {
		for (AbstractOptionsArea area : getAreas()) {
			area.setEnabled(!isArchiveFile());
		}
		super.updateEnablements();
	}

}
