/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.ui;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Composite;

import com.ibm.team.repository.rcp.core.utils.StatusUtil;
import com.ibm.team.scm.client.importz.internal.SCMImportMessages;

/**
 * Page for configuring the the import process itself
 */
public class ConfigureImportProcessingPage extends AbstractImportWizardPage implements IPropertyChangeListener {

	private final ImportConfiguration configuration;
	private ImportEncodingFieldEditor encodingOptions;

	protected ConfigureImportProcessingPage(String pageName, String title,
			ImageDescriptor titleImage, ImportConfiguration configuration) {
		super(pageName, title, titleImage);
		this.configuration = configuration;
	}

	protected void createPageAreas(Composite composite) {
		BaselineOptions baselineOptions = new BaselineOptions(composite, configuration);
        addArea(baselineOptions);
        encodingOptions = new ImportEncodingFieldEditor(composite, configuration);
        encodingOptions.setPropertyChangeListener(new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				updateEnablements();
			}
		});
        ReconcileOptionsArea reconcileOptions = new ReconcileOptionsArea(composite, configuration);
        addArea(reconcileOptions);
	}

	protected void initControls() {
        encodingOptions.load();
        super.initControls();
	}
	
	@Override
	protected IStatus validatePageState() {
		if (!encodingOptions.isValid()) {
			return StatusUtil.newStatus(this, IStatus.ERROR, SCMImportMessages.ConfigureImportPage_0);
		}
		return super.validatePageState();
	}

	@Override
	protected ImportConfiguration getConfiguration() {
		return configuration;
	}
	
	@Override
	public void updateEnablements() {
		for (AbstractOptionsArea area : getAreas()) {
			area.setEnabled(getConfiguration().isImportIntoWorkspace());
		}
		encodingOptions.setEnabled(getConfiguration().isImportIntoWorkspace(), null);
		super.updateEnablements();
	}
	
    @Override
    public void dispose() {
    	super.dispose();
    	getConfiguration().removePropertyChangeListener(this);
    }

	public void storeValues() {
		encodingOptions.store();
	}
	
	@Override
	protected String getHelpContextId() {
		return IHelpContextIds.HELP_CONTEXT_CONFIGURE_IMPORT_PAGE;
	}

}
