/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.ui;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.runtime.Assert;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.SubMonitor;
import org.eclipse.swt.widgets.Display;

import com.ibm.team.repository.client.IContributorManager;
import com.ibm.team.repository.client.ITeamRepository;
import com.ibm.team.repository.common.IContributor;
import com.ibm.team.repository.common.ItemNotFoundException;
import com.ibm.team.repository.common.TeamRepositoryException;
import com.ibm.team.repository.rcp.core.utils.StatusUtil;
import com.ibm.team.repository.rcp.ui.internal.viewers.ISetWithListeners;
import com.ibm.team.repository.rcp.ui.internal.viewers.SlowFunction;
import com.ibm.team.repository.rcp.ui.operations.IOperationRunner;
import com.ibm.team.repository.rcp.ui.operations.OperationFailedException;
import com.ibm.team.scm.client.importz.internal.SCMImportMessages;

class FetchContributorsFunction extends SlowFunction<Author2ContributorMapping, Author2ContributorMapping> {

	private final ITeamRepository repository;

	public FetchContributorsFunction(
			ITeamRepository repository,
			ISetWithListeners<Author2ContributorMapping> domain,
			IOperationRunner runner) {
		super(domain, runner, Display.getCurrent(), false);
		Assert.isNotNull(repository);
		this.repository = repository;
	}

	@Override
	protected Map<Author2ContributorMapping, Author2ContributorMapping> doFetch(
			Collection<Author2ContributorMapping> toFetch,
			IProgressMonitor reporter) throws OperationFailedException {
		return fetchContributors(repository, toFetch, reporter);
	}

	public static Map<Author2ContributorMapping, Author2ContributorMapping> fetchContributors(
			ITeamRepository repository,
			Collection<Author2ContributorMapping> toFetch,
			IProgressMonitor reporter) {
		Map<Author2ContributorMapping, Author2ContributorMapping> result = new HashMap<Author2ContributorMapping, Author2ContributorMapping>();
		Map<String, List<Author2ContributorMapping>> fetch = new HashMap<String, List<Author2ContributorMapping>>();
		for (Author2ContributorMapping mapping : toFetch) {
			result.put(mapping, mapping);
			if (mapping.getContributor() == null) {
				List<Author2ContributorMapping> list = fetch.get(mapping.getContributorId());
				if (list == null) {
					list = new ArrayList<Author2ContributorMapping>(1);
					fetch.put(mapping.getContributorId(), list);
				}
				list.add(mapping);
			}
		}
		if (!fetch.isEmpty()) {
			Collection<IContributor> contributors = fetchContributorsById(repository, fetch.keySet(), reporter);
			for (IContributor contributor : contributors) {	
				List<Author2ContributorMapping> list = fetch.get(contributor.getUserId());
				if (list != null) {
					for (Author2ContributorMapping mapping : list) {
						mapping.setContributor(contributor);
					}
				}
			}
		}
		return result;
	}

	private static Collection<IContributor> fetchContributorsById(
			ITeamRepository repository,
			Set<String> contributorIds, 
			IProgressMonitor reporter) {
		List<IContributor> result = new ArrayList<IContributor>(contributorIds.size());
		IContributorManager contributorManager = repository.contributorManager();
		SubMonitor monitor = SubMonitor.convert(reporter, SCMImportMessages.FetchContributorsFunction_0, contributorIds.size() * 100);
		for (String id : contributorIds) {
			try {
				IContributor contributor = contributorManager.fetchContributorByUserId(id, monitor.newChild(100));
				if (contributor != null)
					result.add(contributor);
			} catch (ItemNotFoundException e) {
				// Ignore
			} catch (TeamRepositoryException e) {
				StatusUtil.log("com.ibm.team.scm.client.importz", e); //$NON-NLS-1$
			}
		}
		return result;
	}

	@Override
	protected Author2ContributorMapping getDefaultValue() {
		return null;
	}

	@Override
	protected String getOperationName() {
		return SCMImportMessages.FetchContributorsFunction_2;
	}

	public ITeamRepository getRepository() {
		return repository;
	}
	
}