/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.ui;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.osgi.util.NLS;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

import com.ibm.team.scm.client.importz.internal.SCMImportMessages;

public class FolderMappingOptions extends AbstractOptionsArea {

	private Group mappingGroup;
	private Button mapToTopLevelOption;
	private Button mapToComponentOption;
	private boolean mapFolderToComponent;
	private final ImportConfiguration configuration;

	public FolderMappingOptions(Composite parent, final ImportConfiguration configuration) {
        this.configuration = configuration;
		mappingGroup = createGroup(parent, NLS.bind(SCMImportMessages.FolderMappingOptions_0, configuration.getSourceRepositoryKind()), 1);
        
        mapToTopLevelOption = new Button(mappingGroup, SWT.RADIO);
        mapToTopLevelOption.setText(NLS.bind(SCMImportMessages.FolderMappingOptions_1, configuration.getSourceRepositoryKind()));
        mapToTopLevelOption.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, false));
        
        mapToComponentOption = new Button(mappingGroup, SWT.RADIO);
        mapToComponentOption.setText(NLS.bind(SCMImportMessages.FolderMappingOptions_2, configuration.getSourceRepositoryKind()));
        mapToComponentOption.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, false));
        
        mapToTopLevelOption.setSelection(!mapFolderToComponent);
        mapToComponentOption.setSelection(mapFolderToComponent);
        
        mapToTopLevelOption.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
            	setMapFolderToComponent(mapToComponentOption.getSelection());
            }
        });
        mapToComponentOption.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
            	setMapFolderToComponent(mapToComponentOption.getSelection());
            }
        });
        
        final IPropertyChangeListener propertyChangeListener = new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				updateEnablements();
			}
		};
		configuration.addPropertyChangeListener(propertyChangeListener);
        parent.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				configuration.removePropertyChangeListener(propertyChangeListener);
			}
		});
    }

	public void initControls() {
		setEnabled(true);
		mapToTopLevelOption.setSelection(!isMapFolderToComponent());
		mapToComponentOption.setSelection(isMapFolderToComponent());
	}

	public boolean isMapFolderToComponent() {
		return configuration.isMapFolderToComponent();
	}
	
    public void setMapFolderToComponent(boolean mapFolderToComponent) {
    	if (mapFolderToComponent != configuration.isMapFolderToComponent())
    		configuration.setMapFolderToComponent(mapFolderToComponent);
	}
	
	protected boolean isSingleFolderSelected() {
		return configuration.isSingleFolderSelected();
	}

	public void updateEnablements() {
		mapToTopLevelOption.setEnabled(isEnabled());
		mapToComponentOption.setEnabled(isEnabled() && isSingleFolderSelected());
	}
}
