/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.ui;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.MultiStatus;
import org.eclipse.core.runtime.Status;

import com.ibm.team.repository.rcp.core.utils.StatusUtil;
import com.ibm.team.repository.rcp.ui.internal.utils.JFaceUtils;
import com.ibm.team.scm.client.importz.internal.SCMImportMessages;

public class ImportLoggingHelper {
	
	public static final String PLUGIN_ID = "com.ibm.team.scm.client.importz"; //$NON-NLS-1$
	
    public static void logException(String title, Throwable t) {
    	if (t instanceof CoreException) {
        	StatusUtil.log(new MultiStatus(PLUGIN_ID, IStatus.ERROR, new IStatus[] { ((CoreException)t).getStatus() },
        			title, t));
    	} else {
    		StatusUtil.log(new Status(IStatus.ERROR, PLUGIN_ID, 0,
                    title + ": " + (t.getLocalizedMessage() == null ? "" : t.getLocalizedMessage()), t)); //$NON-NLS-1$ //$NON-NLS-2$
    	}
    }
    
    public static void displayError(String title, Throwable t) {
        logException(title, t);
        
        String message = t.getLocalizedMessage() == null ? SCMImportMessages.ImportLoggingHelper_3 : t.getLocalizedMessage();
        
        final int MAX_LENGTH = 500;
        if (message.length() > MAX_LENGTH) {
            message = message.substring(0, MAX_LENGTH) + "..."; //$NON-NLS-1$
        }
        
        doDisplayError(title, message);
    }
    
    public static void displayError(String title, String message) {
        StatusUtil.log(new Status(IStatus.ERROR, PLUGIN_ID, 0,
                title + ": " + message, null)); //$NON-NLS-1$
        doDisplayError(title, message);
    }
    
    private static void doDisplayError(String title, String message) {       
    	JFaceUtils.showMessage(title, message, IStatus.ERROR);
    }
}
