/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.ui;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

import com.ibm.team.scm.client.importz.internal.SCMImportMessages;

public class ImportTargetOptions extends AbstractOptionsArea {
	private Button importToWorkspaceOption;
	private Button importToArchiveOption;
	private ImportConfiguration configuration;
	private Group importGroup;
    
    public ImportTargetOptions(Composite parent, ImportConfiguration importConfiguration) {
        this.configuration = importConfiguration;
        
        importGroup = createGroup(parent, SCMImportMessages.ImportTargetOptions_0, 1);

        importToWorkspaceOption = new Button(importGroup, SWT.RADIO);
        importToWorkspaceOption.setText(SCMImportMessages.ImportTargetOptions_1);
        importToWorkspaceOption.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, false));
        importToWorkspaceOption.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
            	configuration.setImportIntoWorkspace(importToWorkspaceOption.getSelection());
            }
        });
        
        importToArchiveOption = new Button(importGroup, SWT.RADIO);
        importToArchiveOption.setText(SCMImportMessages.ImportTargetOptions_2);
        importToArchiveOption.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, false));
        importToArchiveOption.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
            	configuration.setImportIntoWorkspace(importToWorkspaceOption.getSelection());
            }
        });       
        final IPropertyChangeListener propertyChangeListener = new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				updateEnablements();
			}
		};
		configuration.addPropertyChangeListener(propertyChangeListener);
        parent.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				configuration.removePropertyChangeListener(propertyChangeListener);
			}
		});
    }

    public void initControls() {
        importToWorkspaceOption.setSelection(configuration.isImportIntoWorkspace());
        importToArchiveOption.setSelection(!configuration.isImportIntoWorkspace());
        updateEnablements();
    }

    protected void updateEnablements() {
    	importToWorkspaceOption.setEnabled(isEnabled() && configuration.isImportFullHistory());
        importToArchiveOption.setEnabled(isEnabled() && configuration.isImportFullHistory());
    }
    
}
