/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.ui;

import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;

import com.ibm.team.scm.client.importz.internal.SCMImportMessages;

public class ReconcileOptionsArea extends AbstractOptionsArea {

	public static final String RECONCILE_WITH_EXISTING = "com.ibm.team.scm.client.importz.internal.ui.reconcileWithExisting"; //$NON-NLS-1$
	
	private ImportConfiguration configuration;
	private Group group;
	private Button reconcileButton;

	public ReconcileOptionsArea(Composite parent,
			ImportConfiguration configuration) {
		this.configuration = configuration;
		
		
		group = createGroup(parent, SCMImportMessages.ReconcileOptionsArea_1, 1);
		
        reconcileButton = new Button(group, SWT.CHECK);
        reconcileButton.setText(SCMImportMessages.ReconcileOptionsArea_0);
        reconcileButton.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				ReconcileOptionsArea.this.configuration.setProperty(RECONCILE_WITH_EXISTING, (reconcileButton.getSelection()));
			}
        });
    	GridDataFactory.defaultsFor(reconcileButton)
    		.grab(true, false)
    		.span(1, 1)
    		.applyTo(reconcileButton);
    	
        final IPropertyChangeListener propertyChangeListener = new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				updateEnablements();
			}
		};
		configuration.addPropertyChangeListener(propertyChangeListener);
        parent.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				ReconcileOptionsArea.this.configuration.removePropertyChangeListener(propertyChangeListener);
			}
		});
	}

	@Override
	public void initControls() {
		super.initControls();
		reconcileButton.setSelection(((Boolean)configuration.getProperty(RECONCILE_WITH_EXISTING, Boolean.FALSE)).booleanValue());
	}
	
	@Override
	protected void updateEnablements() {
		// Always enable since it is difficult to determine if we selected an existing component or not
		// due to our use of the TEamPlaceSelectionPage
		reconcileButton.setEnabled(isEnabled());
	}

}
