/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.ui;

import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.ibm.team.scm.client.importz.internal.SCMImportMessages;

public abstract class StatisticsArea extends AbstractOptionsArea {

	private Group statisticsGroup;
	private Button showStatButton;
	private Text textField;

	public StatisticsArea(Composite parent, final ImportConfiguration configuration) {
		statisticsGroup = createGroup(parent, SCMImportMessages.StatisticsArea_0, 2, true);
        
        textField = new Text(statisticsGroup, SWT.MULTI | SWT.READ_ONLY);
		GridData data = new GridData(GridData.FILL, GridData.FILL, true, true);
        data.horizontalSpan = 2;
        textField.setLayoutData(data);
        final IPropertyChangeListener propertyChangeListener = new IPropertyChangeListener() {
			public void propertyChange(PropertyChangeEvent event) {
				if (event.getProperty().equals(ImportConfiguration.ARCHIVE_FILENAME)) {
					setStatistics(""); //$NON-NLS-1$
				}
			}
		};
		configuration.addPropertyChangeListener(propertyChangeListener);
        parent.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				configuration.removePropertyChangeListener(propertyChangeListener);
			}
		});
        
		// Add a spacer
		Label label = new Label(statisticsGroup, SWT.NONE);
		label.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, false));
		
        // Add button to calculate statistics
        showStatButton = new Button(statisticsGroup, SWT.PUSH);
        showStatButton.setText(SCMImportMessages.StatisticsArea_2);
        showStatButton.addSelectionListener(new SelectionAdapter() {
            public void widgetSelected(SelectionEvent e) {
                showStatistics();
            }
        });
        showStatButton.setLayoutData(new GridData(GridData.FILL, GridData.FILL, false, false));
	}

	protected abstract void showStatistics();

	public void setStatistics(String stats) {
		if (stats == null) {
			textField.setText(""); //$NON-NLS-1$
		} else {
			textField.setText(stats);
		}
	}
}
