/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.utils;

import java.io.File;
import java.io.IOException;
import java.util.Date;

import org.eclipse.core.runtime.Platform;
import org.eclipse.osgi.util.NLS;

import com.ibm.team.repository.rcp.core.utils.StatusUtil;
import com.ibm.team.scm.client.importz.internal.SCMImportMessages;

public class DebugUtils {
	
	private static final String PLUGIN_ID = "com.ibm.team.scm.client.importz"; //$NON-NLS-1$
	public static boolean DEBUG_IMPORT = false;
	public static boolean DEBUG_TIMINGS = false;
	public static String DEBUG_TIMINGS_LOG_FILE = null;
	public static boolean DEBUG_LOG_FILES = false;
	public static boolean ALWAYS_INITIALIZE_IMPORT_TREE = false;
	
	static {
		if ("true".equalsIgnoreCase(Platform.getDebugOption(PLUGIN_ID + "/debug"))) { //$NON-NLS-1$ //$NON-NLS-2$
			DEBUG_IMPORT = "true".equalsIgnoreCase(Platform.getDebugOption(PLUGIN_ID + "/import")); //$NON-NLS-1$ //$NON-NLS-2$
			DEBUG_TIMINGS = "true".equalsIgnoreCase(Platform.getDebugOption(PLUGIN_ID + "/timings")); //$NON-NLS-1$ //$NON-NLS-2$
			DEBUG_TIMINGS_LOG_FILE = Platform.getDebugOption(PLUGIN_ID + "/timingsLogFile"); //$NON-NLS-1$
			DEBUG_LOG_FILES = "true".equalsIgnoreCase(Platform.getDebugOption(PLUGIN_ID + "/logFiles")); //$NON-NLS-1$ //$NON-NLS-2$
			ALWAYS_INITIALIZE_IMPORT_TREE =  "true".equalsIgnoreCase(Platform.getDebugOption(PLUGIN_ID + "/clearImportTree")); //$NON-NLS-1$ //$NON-NLS-2$
		}
	}
	
	public static File createTempFile(String name) throws IOException {
		File directory = null;
		if (DebugUtils.DEBUG_LOG_FILES) {
			Date d = new Date();
			directory = new File(System.getProperty("user.home"), "importDebugData" + (d.getYear() + 1900) + (d.getMonth() + 1) + d.getDate()); //$NON-NLS-1$ //$NON-NLS-2$
			if (!directory.exists())
				directory.mkdir();
		}
		// The prefix must be 3 characters long
		if (name.length() < 3) {
			name += "XXX"; //$NON-NLS-1$
		}
		File file = null;
		try {
			file = File.createTempFile(name, null, directory);
		} catch (IOException e) {
			// The problem could be that the name contains a colon on Windows
			if (name.indexOf(':') != -1 || name.indexOf('\\') != -1) {
				String newName = name.replaceAll(":", "_"); //$NON-NLS-1$ //$NON-NLS-2$
				newName = newName.replaceAll("\\\\", "_"); //$NON-NLS-1$ //$NON-NLS-2$
				newName = newName.replaceAll("\\*", "_"); //$NON-NLS-1$ //$NON-NLS-2$
				try {
					file = File.createTempFile(newName, null, directory);
				} catch (IOException e1) {
					// That didn't work either so we'll fall through and report the original exception
				}
			}
			if (file == null) {
				String message;
				if (directory == null) {
					message = NLS.bind(SCMImportMessages.DebugUtils_0, name);
				} else {
					message = NLS.bind(SCMImportMessages.DebugUtils_1, name, directory.getAbsolutePath());
				}
				IOException ex = new IOException(message);
				ex.initCause(e);
				throw ex;
			}
		}
		if (!DebugUtils.DEBUG_LOG_FILES) {
			file.deleteOnExit();
		}
		return file;
	}

	public static void deleteTempFile(File file) {
		if (!DebugUtils.DEBUG_LOG_FILES && file != null) {
			if (!file.delete()) {
				// let's log the fact that we couldn't delete the file
				StatusUtil.log("com.ibm.team.scm.client.importz", NLS.bind("Could not delete temp file ''{0}''", file.getAbsolutePath())); //$NON-NLS-1$ //$NON-NLS-2$
			}
		}
	}

}
