/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp. 
 *******************************************************************************/
package com.ibm.team.scm.client.importz.internal.utils;

import java.io.IOException;
import java.io.OutputStream;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;

import com.ibm.team.scm.client.importz.internal.SCMImportMessages;

public class ImportProgressMonitorOutputStream extends ProgressMonitorOutputStream {
    private int progress;
    private IProgressMonitor monitor;
    
    public ImportProgressMonitorOutputStream(OutputStream out, long bytesTotal, int updateIncrement,
            IProgressMonitor monitor) {
        super(out, bytesTotal, updateIncrement, monitor);
        this.monitor = monitor;
        monitor.beginTask(SCMImportMessages.ImportProgressMonitorOutputStream_0, 100);
    }

    protected void updateMonitor(long bytesRead, long size, IProgressMonitor monitor) {
        if (monitor.isCanceled()) throw new OperationCanceledException();
        int percent = (int)(bytesRead * 100 / size);
        monitor.worked(percent - progress);
        progress = percent;
    }
    
    public void close() throws IOException {
        try {
            super.close();
        } finally {
            monitor.done();
        }
    }

}
