/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/ 
package com.ibm.team.scm.client.importz.internal.utils;

import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.InterruptedIOException;
import java.io.OutputStream;

import org.eclipse.core.runtime.IProgressMonitor;

public abstract class ProgressMonitorOutputStream extends FilterOutputStream {
    private IProgressMonitor monitor;
    private long bytesTotal;
    private int updateIncrement;
    private long bytesWritten;
    private long nextUpdate;
    
    public ProgressMonitorOutputStream(OutputStream out, long bytesTotal, int updateIncrement,
                            IProgressMonitor monitor) {
        super(out);
        this.bytesTotal = bytesTotal;
        this.updateIncrement = updateIncrement;
        this.nextUpdate = updateIncrement;
        this.monitor = monitor;
        update(true);
    }
    
    public void write(int b) throws IOException {
        super.write(b);
        bytesWritten++;
        update(false);
    }
    
    public void write(byte[] b, int off, int len) throws IOException {
        try {
            out.write(b, off, len);
            bytesWritten += len;
            update(false);
        } catch (InterruptedIOException e) {
            bytesWritten += e.bytesTransferred;
            update(false);
            throw e;
        }
    }
    
    public void close() throws IOException {
        try {
            super.close();
        } finally {
            update(true);
        }
    }
    
    private void update(boolean force) {
        if (force || bytesWritten >= nextUpdate) {
            if (bytesWritten >= nextUpdate) {
                nextUpdate = bytesWritten + updateIncrement;
            }
            updateMonitor(bytesWritten, bytesTotal, monitor);
        }
    }
    
    protected abstract void updateMonitor(long bytesWritten, long size, IProgressMonitor monitor);
}
